"""
..module:: WaveformFittingMultiproc
        :synopsis: Calculate the Waveform Fitting for each Moho depth in parallel
.. moduleauthor:: Stephen Akinremi <s.akinremi@utwente.nl> Islam Fadel <i.e.a.m.fadel@utwente.nl> (October 2023)
"""

import numpy as np
import os 
from os.path import exists
from os import mkdir
from glob import glob
from rfsed.rf import read_rf, RFStream
import matplotlib.pyplot as plt
from rfsed.SynRF.FwdRF import SynRF
import math
from obspy.signal.tf_misfit import pg
import matplotlib.pyplot as plt
from matplotlib.legend_handler import HandlerLine2D as HL 
from sklearn.metrics import mean_squared_error
import matplotlib.gridspec as gridspec
import matplotlib.transforms as mtransforms
import multiprocessing
import itertools

#----------------------------------------------------------
def WaveformPara(rfstream, Sedthick, VpSed, VpCrust, rayp, KMoho, HMoho, gauparameter):
    """
    Calculate the Model Parameters for the Waveform Fitting

    :param rfstream: Stream of Receiver Functions
    :type rfstream: obspy Stream
    :param Sedthick: Sediment Layer Thickness
    :type Sedthick: float
    :param VpSed: P-wave velocity in the Sediment
    :type VpSed: float
    :param VpCrust: P-wave velocity in the Crust
    :type VpCrust: float
    :param rayp: Ray Parameter (s/km)
    :type rayp: float
    :param KMoho: numpy array of Vp/Vs ratios for the Moho
    :type KMoho: numpy array    
    :param HMoho: numpy array of Moho depths
    :type HMoho: numpy array
    :param gauparameter: Gaussian Parameter used in the receiver function calculation (e.g. 1.25)
    :type gauparameter: float
    :return: Dictionary of Model Parameters
    """
    VsSed=0.7858 - 1.2344*VpSed + 0.7949*VpSed**2 - 0.1238*VpSed**3 + 0.0064*VpSed**4
    SedDen=((1.6612*(VpSed)) - (0.4721 * ((VpSed)**2)) + (0.0671* ((VpSed)**3)) - (0.0043* ((VpSed)**4)) + (0.000106* ((VpSed)**5)))
    VsCrust=0.7858 - 1.2344*VpCrust + 0.7949*VpCrust**2 - 0.1238*VpCrust**3 + 0.0064*VpCrust**4
    CrustDen=((1.6612*(VpCrust)) - (0.4721 * ((VpCrust)**2)) + (0.0671* ((VpCrust)**3)) - (0.0043* ((VpCrust)**4)) + (0.000106* ((VpCrust)**5)))
    #----------------------------------------------------------
    # RF Stream 
    delta = rfstream[0].stats.delta
    b = rfstream[0].stats.starttime - rfstream[0].stats.onset
    t = (np.arange(0, len(rfstream[0].data), 1) * delta) + b
    staname = rfstream[0].stats.station
    Stacked=rfstream.select(component='R').stack()
    StackedRF=Stacked[0]
    StackedData=StackedRF.data
    Stdd=np.std(StackedData, axis=0)
    StddPlus = StackedData + Stdd
    StddMinus = StackedData - Stdd
    delay=abs(t[0])
    n=len(StackedData)
    # Waveform Fitting start from t = 0 to t = 25 seconds
    tzero=np.where(t==0)[0]
    tzero=int(tzero)  
    start=tzero
    end= 25/delta
    end=tzero + int(end)
    #----------------------------------------------------------
    parameters=[{'rfstream':rfstream, 'rayp': rayp, 'gauparameter':gauparameter, 'Sedthick':Sedthick,  'VpSed':VpSed, 'VpCrust':VpCrust, 'VsSed':VsSed, 'SedDen':SedDen, 'VsCrust':VsCrust,\
            'CrustDen':CrustDen, 'HMoho': HMoho, 'KMoho': KMoho, 'delta':delta, 'time':t, 'staname':staname, 'StackedData':StackedData, 'Stdd':Stdd, \
            'StddPlus':StddPlus, 'StddMinus':StddMinus, 'delay': delay, 'n':n, 'tstart': start, 'tend':end}]
    return parameters
#----------------------------------------------------------
def WaveformFit_multiproc(inputparams):
    """
    Calculate the Waveform Fitting for each Moho depth

    :param inputparams: List of input parameters (HMoho, OtherParams)
    :type inputparams: list
    :return: List of Waveform Fitting Results
    """
    HMoho, OtherParams = inputparams
    HMoho=HMoho[0]
    OtherParams=OtherParams[0]
    Sedthick=OtherParams['Sedthick']
    VpSed=OtherParams['VpSed']
    VsSed=OtherParams['VsSed']
    SedDen=OtherParams['SedDen']
    VpCrust=OtherParams['VpCrust']
    VsCrust=OtherParams['VsCrust']
    CrustDen=OtherParams['CrustDen']
    rayp=OtherParams['rayp']
    KMoho=OtherParams['KMoho']
    delta=OtherParams['delta']
    delay=OtherParams['delay']
    t=OtherParams['time']
    start=OtherParams['tstart']
    end=OtherParams['tend']
    n=OtherParams['n']
    StackedData=OtherParams['StackedData']
    gaussian=OtherParams['gauparameter']
    MohoSol=[]
    for i in range(len(KMoho)):
        Ktemp = KMoho[i]
        Htemp = HMoho
        #----------------------------------------------------------
        # Calculate the model parameters
        #----------------------------------------------------------
        depth = np.array([Sedthick, Htemp, 77.5])
        vp = np.array([VpSed, VpCrust, 8.045])
        vs = np.array([VsSed, (VpCrust/Ktemp), 4.49])
        rho = np.array([SedDen, CrustDen, 3.299])
        Synth=SynRF(depth, vp, vs, rho, rayp, dt=delta, npts=n, ipha=1)
        Synth.run_fwd()
        Synth.filter(freqmin=0.05, freqmax=1.25, order=2, zerophase=True)
        rf_synth=Synth.run_deconvolution(pre_filt=[0.05, 1.25], shift=delay, gaussian=gaussian)
        rf_synth=(rf_synth[0]).data

        StackedData = StackedData
        #----------------------------------------------------------
        # Goodness of Fit Evaluation
        #----------------------------------------------------------
        # general constants for goodness of fit evalaution
        dt = delta
        fmin = .5
        fmax = 10
        nf = 100
        n=n
        #Subset the Real and Synthetic Data to 0 to 25 seconds
        RealData = StackedData[start:end]
        SynthData = rf_synth[start:end]
        t=t[start:end]
        #----------------------------------------------------------
        st1a= RealData
        st2= SynthData  # st2 is the reference data
        #----------------------------------------------------------
        # Correlation Coefficient
        Correlation = np.corrcoef(RealData, SynthData)
        Correlation = Correlation[0,1]
        #----------------------------------------------------------
        # RMSE Criteria
        RMSE = math.sqrt(mean_squared_error(RealData, SynthData))
        #----------------------------------------------------------
        # Normalise Phase Goodness of Fit Criteria (PG)
        abs_PG=abs(RealData)
        abs_rf=abs(SynthData)
        PG_Eq=np.where(abs_PG == 0, abs_PG, RealData/abs_PG)
        Synth_Eq=np.where(abs_rf == 0, abs_rf, SynthData/abs_rf)
        PG=pg(PG_Eq, Synth_Eq, dt=dt, fmin=fmin, fmax=fmax, nf=nf, w0=6, st2_isref=True, a=10., k=1.)
        #----------------------------------------------------------
        MohoSol.append([Correlation, RMSE, PG, Htemp, Ktemp])
    return MohoSol
#----------------------------------------------------------
def run_waveformfitting(nproc, HMoho, ModelParams):
    """
    Run the Waveform Fitting Function in parallel for each Moho depth

    :param nproc: Number of processors to use
    :type nproc: int
    :param HMoho: numpy array of Moho depths
    :type HMoho: numpy array
    :param ModelParams: Dictionary of Model Parameters from function WaveformPara
    :type ModelParams: dict
    :return: List of Waveform Fitting Results
    """
    nproc = nproc
    HMoho = HMoho
    with multiprocessing.Pool(nproc) as pool:
        inputparams=((inputparams, ModelParams) for inputparams in itertools.product(HMoho))
        WaveformFitResults=pool.map(WaveformFit_multiproc, inputparams)
    pool.close()
    pool.join()
    return WaveformFitResults
#----------------------------------------------------------
def plotbestmodel(WaveformResults, ModelParams, wtCorr, wtRMSE, wtPG, savepath, format):
    """
    Plot the best model from the Waveform Fitting Results

    :param WaveformResults: List of Waveform Fitting Results
    :type WaveformResults: list
    :param ModelParams: Dictionary of Model Parameters
    :type ModelParams: dict
    :param wtCorr: Weighting for the Correlation Coefficient
    :type wtCorr: float
    :param wtRMSE: Weighting for the Root-mean-square Error
    :type wtRMSE: float
    :param wtPG: Weighting for the Phase Goodness of Fit
    :type wtPG: float
    :param savepath: Path to save the plot
    :type savepath: str
    :param format: Format to save the plot
    :type format: str
    """
    HSed=ModelParams[0]['Sedthick']
    VpSed=ModelParams[0]['VpSed']
    VsSed=ModelParams[0]['VsSed']
    SedDen=ModelParams[0]['SedDen']
    HMoho=ModelParams[0]['HMoho']
    KMoho=ModelParams[0]['KMoho']
    VpCrust=ModelParams[0]['VpCrust']
    VsCrust=ModelParams[0]['VsCrust']
    CrustDen=ModelParams[0]['CrustDen']
    rayp=ModelParams[0]['rayp']
    delta=ModelParams[0]['delta']
    delay=ModelParams[0]['delay']
    staname=ModelParams[0]['staname']
    gaussian=ModelParams[0]['gauparameter']
    rfstream=ModelParams[0]['rfstream']
    if wtCorr==None: wtCorr=0.4
    if wtRMSE==None: wtRMSE=0.4
    if wtPG==None: wtPG=0.2
    AllSolutions = np.zeros(((len(HMoho)*len(KMoho)), 5))
    z=0
    for i in WaveformResults:
        for j in i:
            # print(j)
            AllSolutions[z,:] = [j[0], j[1], j[2], j[3], j[4]]
            z=z+1
    #----------------------------------------------------------
    # Normalise the Correlation and RMSE and PG values 
    #----------------------------------------------------------
    Corr=AllSolutions[:,0]
    MaxCorr=max(Corr)
    RMSE=AllSolutions[:,1]
    MaxRSME=max(RMSE)
    PG=AllSolutions[:,2]
    MaxPG=max(PG)
    NormCorr=Corr/MaxCorr
    NormRMSE=RMSE/MaxRSME
    NormPG=PG/MaxPG
    GoF= wtCorr*NormCorr + wtRMSE*(1-NormRMSE) + wtPG*NormPG #Invert the RMSE values
    #-------------------------
    Htemp=AllSolutions [:,3]
    Ktemp=AllSolutions [:,4]
    #-------------------------
    AllCriteria=zip(Corr, RMSE, PG, GoF, Htemp, Ktemp)
    Solutions = np.zeros((len(GoF), 6))
    z=0
    for i in AllCriteria:
        Solutions[z,:] = [i[0], i[1], i[2], i[3], i[4], i[5]]
        z=z+1
    BestModel_Correlation = Solutions[np.argmax(Solutions[:,0]),:]
    BestModel_RMSE = Solutions[np.argmin(Solutions[:,1]),:]
    BestModel_PG = Solutions[np.argmax(Solutions[:,2]),:]
    BestModel_GoF = Solutions[np.argmax(Solutions[:,3]),:]
    print('Best Model from Correlation Coefficient: Moho at %.2f km with Vp/Vs of %.2f'%(BestModel_Correlation[4],BestModel_Correlation[5]))
    print('Best Model from RMSE: Moho at %.2f km with Vp/Vs of %.2f'%(BestModel_RMSE[4],BestModel_RMSE[5]))
    print('Best Model from PG: Moho at %.2f km with Vp/Vs of %.2f'%(BestModel_PG[4],BestModel_PG[5]))
    print('Best Model from Overall Goodness of Fit: Moho at %.2f km with Vp/Vs of %.2f'%(BestModel_GoF[4],BestModel_GoF[5]))
    labels=['Correlation Coefficient', 'Root-mean-square Error', 'Phase Goodness of Fit', 'Overall Goodness of Fit']
    model_criteria=zip(Corr, RMSE, PG, GoF)
    for i in range(len(labels)):
        suff=labels[i]
        if i==0:
            Criteria=Corr
            BestModel=BestModel_Correlation
        elif i==1:
            Criteria=RMSE
            BestModel=BestModel_RMSE
        elif i==2:
            Criteria=PG
            BestModel=BestModel_PG
        elif i==3:
            Criteria=GoF
            BestModel=BestModel_GoF
        # ----------------------------------------------------------
        # Plot the Solutions
        # ----------------------------------------------------------
        plt.figure(figsize=(18, 6)) 
        gs_corr = gridspec.GridSpec(1, 3, width_ratios=[6,8,4])
        minVpVs=min(KMoho)
        maxVpVs=max(KMoho)
        minMoho=min(HMoho)
        maxMoho=max(HMoho)
        ax1 = plt.subplot(gs_corr[0, 0])
        plt.tricontourf(Solutions[:,4], Solutions[:,5], Criteria,50, cmap ="jet")
        cb=plt.colorbar(format='%.3f', orientation="vertical")
        cb.ax.set_xlabel('S', rotation=0)
        p1,=ax1.plot(BestModel[4],BestModel[5], 'k+', color='black', mew=2, ms=10, \
        label='%s Best Model %.2f km %.2f Vp/Vs'%(staname,BestModel[4],BestModel[5]+0.001))
        props = dict(boxstyle='round', facecolor='wheat', alpha=0.5)
        plt.legend(bbox_to_anchor=(0., 1.02, 1., .102), loc=3,\
            ncol=2, mode="expand", borderaxespad=0, \
            handler_map={p1:HL(numpoints=1)})
        ax1.set_ylabel('Vp/Vs')
        ax1.set_xlabel('depth (km)')
        ax1.set_ylim(minVpVs, maxVpVs)
        ax1.set_xlim(minMoho, maxMoho)
        #----------------------------------------------------------
        # Plot Stacked Data, Standard Deviation and Synthetic RF (Show time between -5 to 25 seconds)
        #----------------------------------------------------------
        delta = rfstream[0].stats.delta
        b = rfstream[0].stats.starttime - rfstream[0].stats.onset
        t = (np.arange(0, len(rfstream[0].data), 1) * delta) + b
        staname = rfstream[0].stats.station
        Stacked=rfstream.select(component='R').stack()
        StackedRF=Stacked[0]
        StackedData=StackedRF.data
        Stdd=np.std(StackedData, axis=0)
        StddPlus = StackedData + Stdd
        StddMinus = StackedData - Stdd
        n=len(StackedData)
        #----------------------------------------------------------
        H=BestModel[4]
        K=BestModel[5]
        depth = np.array([HSed, H, 77.5])
        vp = np.array([VpSed, VpCrust, 8.045])
        vs = np.array([VsSed, (VpCrust/K), 4.49])
        rho = np.array([SedDen, CrustDen, 3.299])
        Synth=SynRF(depth, vp, vs, rho, rayp, dt=delta, npts=n, ipha=1)
        Synth.run_fwd()
        Synth.filter(freqmin=0.05, freqmax=1.25, order=2, zerophase=True)
        rf_synth=Synth.run_deconvolution(pre_filt=[0.05, 1.25], shift=delay, gaussian=gaussian)
        rf_synth=(rf_synth[0]).data
        l = len(rf_synth)
        synth_time = np.arange(0, l)
        synth_time = (delta *  synth_time) - delay 
        # # Plot to visualise difference
        ax2=plt.subplot(gs_corr[0, 1])
        plt.plot(t, StackedData, color = 'black', label = 'Stacked RF')
        plt.plot(synth_time, rf_synth, '--',  color = 'black', label = 'Synthetic RF')
        plt.fill_between(t, StddPlus, StddMinus, alpha = 0.3, color = 'grey', 
                            label = 'RF Data Std' )
        ax2.set_title("%s Best Model" %suff)
        # Subset the Data to -5 to 25 seconds
        ax2.set_xlabel('time (s)')
        ax2.set_ylabel('amplitude')
        ax2.set_yticklabels([])
        ax2.set_yticks([])
        ax2.set_xlim(-5, 25)
        ax2.legend()
        #----------------------------------------------------------
        # Plot Vp and Vs with Depth
        #----------------------------------------------------------
        ax3=plt.subplot(gs_corr[0, 2])
        dep=(0, HSed, HSed, H, H, 50)
        vp=(VpSed, VpSed, VpCrust, VpCrust, 8.045, 8.045)
        vs=(VsSed, VsSed, VpCrust/K, VpCrust/K, 4.49, 4.49)
        ax3.plot(vp, dep, '--', color = 'red', label = 'Vp')
        ax3.plot(vs, dep, ':', color = 'blue', label = 'Vs')
        ax3.xaxis.set_ticks_position('bottom')
        # ax3.yaxis.tick_right()
        ax3.set_ylim(0, 50)                   
        ax3.set_xlim(1, 10)  
        ax3.invert_yaxis()
        ax3.set_ylabel('depth(km)')
        ax3.legend(fontsize=15)
        ax3.set_xlabel("v (km/s)")
        #------------------------------------------
        #    Save Plot to File
        #-------------------------------------------
        filename = "%s/%s_%s.%s"%(savepath, staname, suff, format)
        plt.savefig(filename , format=format, transparent=False, 
                    dpi=300, bbox_inches = 'tight', pad_inches=0.1)
        plt.show()
        plt.close("all")  
#----------------------------------------------------------


#%% Importing the libraries
import numpy as np
from rfsed.SynRF.FwdRF import SynRF
import os 
from rfsed.SynthExp import hkSynth, plothkSynth, hkSeqSynth, plothkSeqSynth, ResonanceFilt, plotfiltSynthrf
from rfsed.util import rfMoho_example, rfSed_example, save_plot

#%% Test the Synthetic HK function
#----------------------------------------------------------
# Synthetic Parameters
moddim = 4
rayp = 0.05826535137278129   #for 60 degree distance
gaussian = 1.25
delay = 10
n = 2001
delta = 0.025
#----------------------------------------------------------
# Model Original
#----------------------------------------------------------
depth = np.array([35, 77.5])
vp = np.array([6.90, 8.045])
vs = np.array([3.951172240000002, 4.49])
rho = np.array([2.939564099940003, 3.299])
Synth=SynRF(depth, vp, vs, rho, rayp, dt=delta, npts=n, ipha=1)
Synth.run_fwd()
Synth.filter(freqmin=0.05, freqmax=1.25, order=2, zerophase=True)
rf=Synth.run_deconvolution(pre_filt=[0.05, 1.25], shift=delay, gaussian=gaussian)
rf=(rf[0]).data
l = len(rf)
t = np.arange(0, l)
t = (delta *  t) - delay
Vp=vp[0]
K= np.linspace(1.65,1.95,121)
H=np.linspace(20,60,201)
w1, w2, w3 = [0.6, 0.3, 0.1]
savedir=save_plot()
savepath = savedir + '/SynthExp_HK_Result/'
if not os.path.exists(savepath):  # create data folder if necessary
    os.mkdir(savepath)
#-------------------------------------------------------------------------
HKResultSynth=hkSynth(Synthrf=rf, time=t, rayp=rayp, w1 = w1, w2 = w2, w3 = w3, K= K, H=H, Vp = Vp, layer = None)
plothkSynth(HKResultSynth, g = [75.,10., 15., 2.5], rmneg = None, format = 'jpg', savepath=savepath)

#%% Test the Synthetic Sequential HK function
import numpy as np
from rfsed.SynRF.FwdRF import SynRF
import os 
#----------------------------------------------------------
# Synthetic Parameters
moddim = 4
rayp = 0.05826535137278129   #for 60 degree distance
gaussian_Moho = 1.25
gaussian_Sed = 5
delay = 10
n = 2001
delta = 0.025
#----------------------------------------------------------
# Model Original
#----------------------------------------------------------
depth = np.array([2, 35, 77.5])
vp = np.array([3, 6.90, 8.045])
VpSed=vp[0]
VpCrust=vp[1]
VpMoho=vp[1]
SedH=depth[0]
VsSed=0.7858 - 1.2344*VpSed + 0.7949*VpSed**2 - 0.1238*VpSed**3 + 0.0064*VpSed**4
SedDen=((1.6612*(VpSed)) - (0.4721 * ((VpSed)**2)) + (0.0671* ((VpSed)**3)) - (0.0043* ((VpSed)**4)) + (0.000106* ((VpSed)**5)))
VsCrust=0.7858 - 1.2344*VpCrust + 0.7949*VpCrust**2 - 0.1238*VpCrust**3 + 0.0064*VpCrust**4
CrustDen=((1.6612*(VpCrust)) - (0.4721 * ((VpCrust)**2)) + (0.0671* ((VpCrust)**3)) - (0.0043* ((VpCrust)**4)) + (0.000106* ((VpCrust)**5)))
vs = np.array([VsSed,VsCrust, 4.49])
rho = np.array([SedDen, CrustDen, 3.299])
#----------------------------------------------------------
Synth=SynRF(depth, vp, vs, rho, rayp, dt=delta, npts=n, ipha=1)
Synth.run_fwd()
Synth.filter(freqmin=0.05, freqmax=1.25, order=2, zerophase=True)
rfSynthMoho=Synth.run_deconvolution(pre_filt=[0.05, 1.25], shift=delay, gaussian=gaussian_Moho)
rfSynthMoho=(rfSynthMoho[0]).data
#----------------------------------------------------------
Synth=SynRF(depth, vp, vs, rho, rayp, dt=delta, npts=n, ipha=1)
Synth.run_fwd()
Synth.filter(freqmin=0.05, freqmax=1.25, order=2, zerophase=True)
rfSynthSed=Synth.run_deconvolution(pre_filt=[0.05, 1.25], shift=delay, gaussian=gaussian_Sed)
rfSynthSed=(rfSynthSed[0]).data
#----------------------------------------------------------
l = len(rfSynthMoho)
t = np.arange(0, l)
t = (delta *  t) - delay
w1, w2, w3 = [0.6, 0.3, 0.1]
savedir=save_plot()
savepath = savedir + '/SynthExp_SeqHK_Result/'
if not os.path.exists(savepath):  # create data folder if necessary
    os.mkdir(savepath)
KSed= np.linspace(1.65,2.25,201)
HSed=np.linspace(0,10,201)
w1Sed, w2Sed, w3Sed = [0.6, 0.3, 0.1]
KMoho= np.linspace(1.65,1.95,121)
HMoho=np.linspace(20,60,201)
w1Moho, w2Moho, w3Moho = [0.6, 0.3, 0.1]
#-------------------------------------------------------------------------
SynthSeqHKResult=hkSeqSynth(rfSynthSed=rfSynthSed, rfSynthMoho=rfSynthMoho, time=t, rayp=rayp, 
                             HSed=HSed, KSed=KSed, VpSed=VpSed, w1Sed=w1Sed, w2Sed=w2Sed, w3Sed=w3Sed, 
                             HMoho=HMoho, KMoho=KMoho, VpMoho=VpMoho, w1Moho=w1Moho, w2Moho=w2Moho, 
                             w3Moho=w3Moho, g = [75.,10., 15., 2.5], rmneg = None)
plothkSeqSynth(SynthSeqHKResult, g = [75.,10., 15., 2.5], rmneg = None, format = 'jpg', savepath=savepath)

#%% Test the Synthetic Resonance Filtering
import numpy as np
from rfsed.SynRF.FwdRF import SynRF
import os 
#----------------------------------------------------------
# Synthetic Parameters
moddim = 4
rayp = 0.05826535137278129   #for 60 degree distance
gaussalp = 1.25
delay = 10
n = 2001
delta = 0.025
#----------------------------------------------------------
# Model Parameters
#----------------------------------------------------------
depth = np.array([35, 77.5])
vp = np.array([6.90, 8.045])
vs = np.array([3.951172240000002, 4.49])
rho = np.array([2.939564099940003, 3.299])
Synth=SynRF(depth, vp, vs, rho, rayp, dt=delta, npts=n, ipha=1)
Synth.run_fwd()
Synth.filter(freqmin=0.05, freqmax=1.25, order=2, zerophase=True)
rf=Synth.run_deconvolution(pre_filt=[0.05, 1.25], shift=delay, gaussian=gaussian)
rf=(rf[0]).data
l = len(rf)
t = np.arange(0, l)
t = (delta *  t) - delay
savedir=save_plot()
savepath = savedir + '/SynthExp_ResonanceFlt/'
if not os.path.exists(savepath):  # create data folder if necessary
    os.mkdir(savepath)
RemoveResonance= ResonanceFilt(Synthrf=rf, time=t)
# print(RemoveResonance)
plotfiltSynthrf(RemoveResonance, savepath, format = 'jpg')
#%%


"""
Tests for the rf package.
"""

from pkg_resources import resource_filename
import sys
import unittest

import matplotlib
matplotlib.use('Agg')


def run():
    loader = unittest.TestLoader()
    test_dir = resource_filename('rfsed', 'rf/tests/')
    suite = loader.discover(test_dir)
    runner = unittest.runner.TextTestRunner()
    ret = not runner.run(suite).wasSuccessful()
    sys.exit(ret)



import unittest
import warnings
from rfsed.WaveformFittingMultiproc import WaveformPara, run_waveformfitting, plotbestmodel
import numpy as np
from rfsed.util import rfMoho_example, save_tests
try:
    import matplotlib
except ImportError:
    Matplotlib = None

class TestWaveformFittingMultiproc(unittest.TestCase):
    def test_WaveformPara(self):
        rfstream= rfMoho_example()
        VpSed=3.3
        Sedthick=1.8
        VpCrust = 6.9
        gauparameter=1.25
        savepath=save_tests()
        rayp=0.04
        KMoho= np.linspace(1.65,1.95,121)
        HMoho=np.linspace(20,60,201)
        # KMoho= np.linspace(1.65,1.95,5)
        # HMoho=np.linspace(20,60,5)
        wtCorr, wtRMSE, wtPG = [0.5, 0.3, 0.2] #wtCorr+wtRMSE+wtPG=1.0
        WaveformPara(rfstream, Sedthick, VpSed, VpCrust, rayp, KMoho, HMoho, gauparameter)

    def test_run_waveformfitting(self):
            rfstream= rfMoho_example()
            VpSed=3.3
            Sedthick=1.8
            VpCrust = 6.9
            gauparameter=1.25
            savepath=save_tests()
            rayp=0.04
            KMoho= np.linspace(1.65,1.95,121)
            HMoho=np.linspace(20,60,201)
            # KMoho= np.linspace(1.65,1.95,5)
            # HMoho=np.linspace(20,60,5)
            nproc=25
            wtCorr, wtRMSE, wtPG = [0.5, 0.3, 0.2] #wtCorr+wtRMSE+wtPG=1.0
            ModelParams=WaveformPara(rfstream, Sedthick, VpSed, VpCrust, rayp, KMoho, HMoho, gauparameter)
            run_waveformfitting(nproc, HMoho, ModelParams)

    def test_plotbestmodel(self):
            rfstream= rfMoho_example()
            VpSed=3.3
            Sedthick=1.8
            VpCrust = 6.9
            gauparameter=1.25
            savepath=save_tests()
            rayp=0.04
            KMoho= np.linspace(1.65,1.95,121)
            HMoho=np.linspace(20,60,201)
            # KMoho= np.linspace(1.65,1.95,5)
            # HMoho=np.linspace(20,60,5)
            nproc=25
            format='pdf'
            wtCorr, wtRMSE, wtPG = [0.5, 0.3, 0.2] #wtCorr+wtRMSE+wtPG=1.0
            ModelParams=WaveformPara(rfstream, Sedthick, VpSed, VpCrust, rayp, KMoho, HMoho, gauparameter)
            Results=run_waveformfitting(nproc, HMoho, ModelParams)
            plotbestmodel(Results, ModelParams, wtCorr, wtRMSE, wtPG, savepath, format)
        

def suite():
    return unittest.makeSuite(TestWaveformFittingMultiproc, 'test')


if __name__ == '__main__':
    unittest.main(defaultTest='suite')


# Copyright 2013-2019 Tom Eulenfeld, MIT license
"""
All objects and functions below are after the Tom Eulenfeld's
rf project License below.

The MIT License (MIT)

Copyright (c) 2013-2019 Tom Eulenfeld

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
of the Software, and to permit persons to whom the Software is furnished to do
so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"""
"""
rf: receiver function calculation - batch command line utility
"""

import argparse
from argparse import SUPPRESS
from importlib import import_module
import json
import os
from os.path import join
from pkg_resources import resource_filename
import shutil
import sys

import numpy as np
import obspy
from rfsed.rf.rfstream import read_rf
from rfsed.rf.util import iter_event_data, iter_event_metadata

try:
    from tqdm import tqdm
except ImportError:
    def tqdm():
        return None

try:
    basestring
except NameError:
    basestring = str
IS_PY3 = sys.version_info.major == 3

_TF = '.datetime:%Y-%m-%dT%H:%M:%S'

FNAMES = {
    'Q': join('{root}', '{network}.{station}.{location}',
              '{network}.{station}.{location}_{event_time%s}.QHD' % _TF),
    'SAC': join('{root}', '{network}.{station}.{location}',
                '{network}.{station}.{location}.{channel}_'
                '{event_time%s}.SAC' % _TF),
    'H5': '{root}.h5'}
STACK_FNAMES = {
    'Q': join('{root}', '{network}.{station}.{location}.QHD'),
    'SAC': join('{root}', '{network}.{station}.{location}.{channel}.SAC'),
    'H5': '{root}.h5'}
PROFILE_FNAMES = {
    'Q': '{root}.QHD',
    'SAC': join('{root}', 'profile_{channel[2]}_{box_pos}.SAC'),
    'H5': '{root}.h5'}
PLOT_FNAMES = join('{root}', '{network}.{station}.{location}.{channel}.pdf')
PLOT_PROFILE_FNAMES = join('{root}', 'profile_{channel[2]}.pdf')


class _DummyDateTime(object):

    def __format__(self, *args, **kwargs):
        return '*'


class _DummyUTC(object):

    """Dummy UTCDateTime class returning '*' when formatting."""

    def __init__(self):
        self.datetime = _DummyDateTime()


def _create_dir(filename):
    """
    Create directory of filname if it does not exist.
    """
    head = os.path.dirname(filename)
    if head != '' and not os.path.isdir(head):
        os.makedirs(head)


def write(stream, root, format, type=None):
    """Write stream to one or more files depending on format."""
    format = format.upper()
    if len(stream) == 0:
        return
    fname_pattern = (STACK_FNAMES if type == 'stack' else
                     PROFILE_FNAMES if type == 'profile' else
                     FNAMES)[format]
    fname = fname_pattern.format(root=root, **stream[0].stats)
    _create_dir(fname)
    if format == 'H5':
        stream.write(fname, format, mode='a', ignore=('mseed',))
    elif format == 'Q':
        stream.write(fname, format)
    elif format == 'SAC':
        for tr in stream:
            fname = fname_pattern.format(root=root, **tr.stats)
            tr.write(fname, format)


def iter_event_processed_data(events, inventory, pin, format,
                              yield_traces=False, pbar=None):
    """Iterator yielding streams or traces which are read from disc."""
    for meta in iter_event_metadata(events, inventory, pbar=pbar):
        meta['channel'] = '???'
        if 'event_time' not in meta and format != 'H5':
            meta['event_time'] = _DummyUTC()
        fname = FNAMES[format].format(root=pin, **meta)
        kwargs = {}
        if format == 'H5':
            meta.pop('channel')
            kwargs['readonly'] = meta
        try:
            stream = read_rf(fname, format, **kwargs)
        except Exception:
            pass
        else:
            if yield_traces:
                for tr in stream:
                    yield tr
            else:
                yield stream


def _iter_profile(pin, format):
    fname = PROFILE_FNAMES[format].format(root=pin, box_pos='*', channel='???')
    yield read_rf(fname)


def load_func(modulename, funcname):
    """Load and return function from Python module."""
    sys.path.append(os.path.curdir)
    module = import_module(modulename)
    sys.path.pop(-1)
    func = getattr(module, funcname)
    return func


def init_data(data, client_options=None, plugin=None):
    """Return appropriate get_waveforms function.

    See example configuration file for a description of the options."""
    if client_options is None:
        client_options = {}
    try:
        client_module = import_module('obspy.clients.%s' % data)
    except ImportError:
        client_module = None
    if client_module:
        Client = getattr(client_module, 'Client')
        client = Client(**client_options)

        def get_waveforms(event=None, **args):
            return client.get_waveforms(**args)
    elif data == 'plugin':
        modulename, funcname = plugin.split(':')
        get_waveforms = load_func(modulename.strip(), funcname.strip())
    else:
        from obspy import read
        stream = read(data)

        def get_waveforms(network, station, location, channel,
                          starttime, endtime, event=None):
            st = stream.select(network=network, station=station,
                               location=location, channel=channel)
            st = st.slice(starttime, endtime)
            return st

    def wrapper(**kwargs):
        try:
            return get_waveforms(**kwargs)
        except (KeyboardInterrupt, SystemExit):
            raise
        except Exception as ex:
            seedid = '.'.join((kwargs['network'], kwargs['station'],
                               kwargs['location'], kwargs['channel']))
            msg = 'channel %s: error while retrieving data: %s'
            print(msg % (seedid, ex))

    return wrapper


class ConfigJSONDecoder(json.JSONDecoder):
    """Strip lines from comments."""

    def decode(self, s):
        s = '\n'.join(l.split('#', 1)[0] for l in s.split('\n'))
        return super(ConfigJSONDecoder, self).decode(s)


class ParseError(Exception):
    pass


def run(command, conf=None, tutorial=False, **kw):
    """Create example configuration file and tutorial or load config.

    After that call `run_commands`.
    """
    if command == 'create':
        if conf is None:
            conf = 'conf.json'
        srcs = ['conf.json']
        dest_dir = os.path.dirname(conf)
        dests = [conf]
        if tutorial:
            example_files = ['example_events.xml', 'example_inventory.xml',
                             'example_data.mseed']
            srcs.extend(example_files)
            for src in example_files:
                dests.append(os.path.join(dest_dir, src))
        for src, dest in zip(srcs, dests):
            src = resource_filename('rfsed', 'rf/example/%s' % src)
            shutil.copyfile(src, dest)
        return
    # Load configuration
    if conf in ('None', 'none', 'null', ''):
        conf = None
    if conf and (command != 'print' or
                 kw.get('objects', [''])[0] in ('stations', 'events')):
        try:
            with open(conf) as f:
                conf = json.load(f, cls=ConfigJSONDecoder)
        except ValueError as ex:
            print('Error while parsing the configuration: %s' % ex)
            return
        except IOError as ex:
            print(ex)
            return
        # Populate kwargs with conf, but prefer kwargs
        conf.update(kw)
        kw = conf
    run_commands(command, **kw)


DICT_OPTIONS = ['client_options', 'options', 'rf', 'moveout',
                'boxbins', 'boxes', 'profile', 'plot', 'plot_profile']


def run_commands(command, commands=(), events=None, inventory=None,
                 objects=None, get_waveforms=None, data=None, plugin=None,
                 phase=None, moveout_phase=None,
                 path_in=None, path_out=None, format='Q',
                 newformat=None, **kw):
    """Load files, apply commands and write result files."""
    for opt in kw:
        if opt not in DICT_OPTIONS:
            raise ParseError('Unknown config option: %s' % opt)
    for opt in DICT_OPTIONS:
        default = None if opt == 'boxbins' else {}
        d = kw.setdefault(opt, default)
        if isinstance(d, basestring):
            kw[opt] = json.loads(d)
    if phase is not None:
        kw['options']['phase'] = phase
    if moveout_phase is not None:
        kw['moveout']['phase'] = moveout_phase
    if kw['boxbins'] is not None:
        kw['boxes']['bins'] = np.linspace(*kw['boxbins'])

    try:
        if command == 'calc':
            assert len(commands) < 3
            if len(commands) == 2:
                assert commands[0] != commands[1]
        elif command == 'calc':
            assert len(commands) < 2
    except Exception:
        raise ParseError('calc or moveout command given more than once')

    # Read events and inventory
    try:
        if command in ('stack', 'plot'):
            events = None
        elif command != 'print' or objects[0] == 'events':
            if (not isinstance(events, obspy.Catalog) or
                    not isinstance(events, list) or
                    (len(events) == 2 and isinstance(events[0], basestring))):
                if isinstance(events, basestring):
                    format_ = None
                else:
                    events, format_ = events
                events = obspy.read_events(events, format_)
        if command != 'print' or objects[0] == 'stations':
            if not isinstance(inventory, obspy.Inventory):
                if isinstance(inventory, basestring):
                    format_ = None
                else:
                    inventory, format_ = inventory
                inventory = obspy.read_inventory(inventory, format_)
    except Exception:
        print('cannot read events or stations')
        return
    # Initialize get_waveforms
    if command == 'data':
        try:
            # Initialize get_waveforms
            if get_waveforms is None:
                get_waveforms = init_data(
                    data, client_options=kw['client_options'], plugin=plugin)
        except Exception:
            print('cannot initalize data')
            return
    # Print command
    if command == 'print':
        if objects[0] == 'events':
            print(events.__str__(True))
        elif objects[0] == 'stations':
            print(inventory)
        else:
            from rf.rfstream import RFStream
            stream = sum((read_rf(fname) for fname in objects), RFStream())
            print(stream.__str__(True))
        return
    # Select appropriate iterator
    if command == 'data':
        iter_ = iter_event_data(events, inventory, get_waveforms, pbar=tqdm(),
                                **kw['options'])
    elif command == 'plot-profile':
        iter_ = _iter_profile(path_in, format)
    else:
        yt = command == 'profile'
        iter_ = iter_event_processed_data(
            events, inventory, path_in, format, pbar=tqdm(), yield_traces=yt)
    # Run all commands
    if command == 'convert':
        for stream in iter_:
            write(stream, path_out, newformat)
    elif command == 'plot':
        for stream in iter_:
            channels = set(tr.stats.channel for tr in stream)
            for ch in channels:
                st2 = stream.select(channel=ch)
                fname = PLOT_FNAMES.format(root=path_out, **st2[0].stats)
                _create_dir(fname)
                st2.sort(['back_azimuth'])
                st2.plot_rf(fname, **kw['plot'])
    elif command == 'plot-profile':
        for stream in iter_:
            channels = set(tr.stats.channel for tr in stream)
            for ch in channels:
                st2 = stream.select(channel=ch)
                fname = PLOT_PROFILE_FNAMES.format(root=path_out,
                                                   **st2[0].stats)
                _create_dir(fname)
                st2.plot_profile(fname, **kw['plot_profile'])
    elif command == 'stack':
        for stream in iter_:
            stack = stream.stack()
            write(stack, path_out, format, type='stack')
    elif command == 'profile':
        from rfsed.rf.profile import get_profile_boxes, profile
        boxx = get_profile_boxes(**kw['boxes'])
        prof = profile(iter_, boxx, **kw['profile'])
        write(prof, path_out, format, type='profile')
    else:
        commands = [command] + list(commands)
        for stream in iter_:
            for command in commands:
                if command == 'data':
                    pass
                elif command == 'calc':
                    stream.rf(**kw['rf'])
                elif command == 'moveout':
                    stream.moveout(**kw['moveout'])
                else:
                    raise NotImplementedError
            write(stream, path_out, format)


def run_cli(args=None):
    """Command line interface of rf.

    After parsing call `run`.
    """
    from rfsed.rf import __version__
    msg = ('*Note*: The command line tool is rather low priority in the '
           'development. It might also be depreciated in the future.')
    p = argparse.ArgumentParser(description=__doc__, epilog=msg)
    version = '%(prog)s ' + __version__
    p.add_argument('-v', '--version', action='version', version=version)
    msg = 'Configuration file to load (default: conf.json)'
    p.add_argument('-c', '--conf', default='conf.json', help=msg)

    sub = p.add_subparsers(title='commands', dest='command')
    msg = 'create config file in current directory'
    p_create = sub.add_parser('create', help=msg)
    msg = 'retrieve data for further processing'
    p_data = sub.add_parser('data', help=msg)
    msg = 'calculate receiver functions'
    p_calc = sub.add_parser('calc', help=msg)
    msg = 'perform move out correction'
    p_mout = sub.add_parser('moveout', help=msg)
    msg = 'stack receiver functions'
    p_stack = sub.add_parser('stack', help=msg)
    msg = 'stack receiver functions to profile'
    p_profile = sub.add_parser('profile', help=msg)
    msg = 'convert files to different format'
    p_conv = sub.add_parser('convert', help=msg)
    msg = 'print information about events, stations or waveform files'
    p_print = sub.add_parser('print', help=msg)
    msg = 'plot receiver functions'
    p_plot = sub.add_parser('plot', help=msg)
    msg = 'plot receiver function profile'
    p_plotp = sub.add_parser('plot-profile', help=msg)

    msg = 'create example files for tutorial'
    p_create.add_argument('-t', '--tutorial', help=msg, action='store_true')
    # the default='moveout' is an ugly work-around for
    # http://bugs.python.org/issue27227 and related issue9625
    msg = 'calculate receiver functions, perform moveout correction, optional'
    p_data.add_argument('commands', nargs='*', help=msg,
                        choices=('calc', 'moveout'), default='moveout')
    msg = 'perform also moveout correction'
    p_calc.add_argument('commands', nargs='*', help=msg,
                        choices=('moveout',), default='moveout')
    msg = "one of 'events', 'inventory' or filenames"
    p_print.add_argument('objects', nargs='+', help=msg)

    io = [p_calc, p_mout, p_conv, p_plot, p_stack, p_profile, p_plotp]
    for pp in io:
        msg = 'directory of files (SAC, Q) or basename of file (H5)'
        pp.add_argument('path_in', help=msg)
    io.append(p_data)
    for pp in io:
        msg = 'output directory or output file basename'
        pp.add_argument('path_out', help=msg)

    msg = 'new format (supported: Q, SAC or H5)'
    p_conv.add_argument('newformat', help=msg)

    msg = ('Use these flags to overwrite values in the config file. '
           'See the example configuration file for a description of '
           'these options.')
    g2 = p.add_argument_group('optional config arguments', description=msg)
    features_str = ('events', 'inventory', 'data', 'phase',
                    'moveout-phase', 'format')
    for f in features_str:
        g2.add_argument('--' + f, default=SUPPRESS)
    for f in DICT_OPTIONS:
        g2.add_argument('--' + f.replace('_', '-'), default=SUPPRESS)

    # Get command line arguments and start run
    args = vars(p.parse_args(args))
    # second part of the uggly work-around for argparse issue27227
    if args.get('commands') == 'moveout':
        args['commands'] = []
    try:
        run(**args)
    except ParseError as ex:
        p.error(ex)



# %% Test the ExtractEqMultiproc Function
from glob import glob
import os
from obspy import read
from rfsed.ExtractEqMultiproc import Add_sta_time_2cat, Get_Eqtimes_multiproc, ExtractEq_Multiproc
from rfsed.util import catalog_example, raw_data_example, save_Eq_data

staname='NE301'
stalat=6.183432
stalon=53.48334
datapath= raw_data_example()
datafiles = sorted(glob("%s*.dat"%(datapath)))
savedir=save_Eq_data()
savepath=savedir+'/Extracted_Eq_Data_Multiproc/'
if not os.path.exists(savepath):
    os.mkdir(savepath)
filename = savepath + 'eq_data.h5'
catalogfile=catalog_example()
Request_window=[-50, 150] #Time relative to first P arrival
nproc = 40 #Number of processors to use if parallel processing is used
catalog = Add_sta_time_2cat(catalogfile, stalat, stalon, Request_window)
timewindow=Get_Eqtimes_multiproc(catalog, nproc)
eqdata = ExtractEq_Multiproc(datafiles=datafiles, timewindow=timewindow, nproc=nproc, filename=filename)
#%% Check the number of traces in the file
data_extract=read(filename )
print(len(data_extract), 'Traces in the file')
print(data_extract)


"""
.. module:: hkZhu
        :synopsis: H-K stacking for receiver functions after Zhu and Kanamori (2000)
        Zhu, L., & Kanamori, H. (2000). Moho depth variation in southern california from
        teleseismic receiver functions. Journal of Geophysical Research: Solid Earth, 105 , 
        2969-2980. doi: 10.1029/1999jb900322
.. moduleauthor:: Stephen Akinremi <s.akinremi@utwente.nl> Islam Fadel <i.e.a.m.fadel@utwente.nl> (October 2023)
"""

import numpy as np
import matplotlib.pyplot as plt
import matplotlib.transforms as mtransforms
from matplotlib.legend_handler import HandlerLine2D as HL
import matplotlib.gridspec as gridspec
from obspy.geodetics.base import kilometer2degrees
import seaborn
seaborn.set_style("darkgrid", {"axes.facecolor": ".85"})

#----------------------------------------------------------
def getamp(rfdata, tarray, t):
    """
    Get the amplitude of the receiver function at a specific time

    :param rfdata: receiver function data
    :param tarray: time array
    :param t: time to get the amplitude
    :return: amplitude of the receiver function at time t
    """
    amp = rfdata[(np.abs(tarray - t).argmin())]
    return amp
#----------------------------------------------------------

def hk(rfstream, H=np.linspace(20,60,201), K=np.linspace(1.65,1.95,121), Vp=6.9, 
              w1=0.6, w2=0.3, w3=0.1, layer = None, stack=False):
    """
    Calculate the H-K stacking for a receiver function after Zhu and Kanamori (2000)

    :param rfstream: receiver function stream
    :type rfstream: obspy.core.stream.Stream
    :param H: array of Depth values
    :type H: numpy array
    :param K: array of Vp/Vs values
    :type K: numpy array
    :param Vp: P wave velocity of the layer
    :type Vp: float
    :param w1: weight for the Ps arrival
    :param w2: weight for the PpPs arrival
    :param w3: weight for the PsPs+PpSs arrival
    :type w1, w2, w3: float
    :param layer: layer name ('Moho' or 'Sed')
    :type layer: str
    :param stack: stack the receiver functions before the H-K stacking (default False)
    :type stack: bool
    :return: Dictionary of H-K stacking results
    """
    if Vp is None: Vp = 6.5
    if w1 is None: w1 = 0.6
    if w2 is None: w2 = 0.3
    if w3 is None: w3 = 0.1   
    if layer is None: layer = 'Moho'
    if stack is None: stack = False
    StackedRF=rfstream.stack()
    RFStacked=StackedRF[0].data
    slowness=[]
    for tr in rfstream:
        slow=tr.stats['slowness']
        slowness.append(slow)
    AvgSlow = np.mean(slowness)
    rayp=kilometer2degrees(1) * AvgSlow
    #----------------------------------------------------------
    staname = rfstream[0].stats.station
    comp =  rfstream[0].stats.channel
    rfdata = rfstream[0].data
    delta = rfstream[0].stats.delta
    l = len(rfdata)
    t = np.arange(0, l)
    t = (delta *  t) - 10 
    stk = np.zeros((len(K)*len(H),3))
    dbgvalues = np.zeros((len(K)*len(H)*len(rfstream),9))
    z = 0 ; q =0
    for i in range(len(K)):
        Ktemp = K[i]
        for j in range(len(H)):
            Htemp = H[j]
            s = 0.0
            for tr in rfstream:
                trdata = tr.data
                b = tr.stats.starttime - tr.stats.onset
                delta= tr.stats.delta
                rp = kilometer2degrees(1) * tr.stats['slowness']
                if stack == True:
                    trdata = StackedRF[0].data
                    rp = kilometer2degrees(1) * AvgSlow
                #--------------------------------------
                term1= ((Ktemp/Vp)**2 - rp**2)**0.5
                term2= ((1/Vp)**2 - rp**2)**0.5
                tps = Htemp * (term1 - term2)
                tppps = Htemp * (term1 + term2)
                tpsps = Htemp * 2 * (term1) 
                #--------------------------------------
                ampps = getamp(trdata, t, tps)
                ampppps = getamp(trdata,t, tppps)
                amppsps = getamp(trdata, t, tpsps)
                #--------------------------------------
                stemp = (w1 * ampps) + (w2 * ampppps) - (w3 * amppsps)
                s = stemp + s
                dbgvalues[q, :] =[rp, Htemp, Ktemp, tps, tppps, tpsps,\
                ampps, ampppps, amppsps]
                q =q + 1
            stk[z,:] = [Htemp, Ktemp, s]
            z = z + 1
    bmod = stk[np.argmax(stk[:,2]),:]
    #Show the best model
    plt.tricontourf(stk[:,0], stk[:,1], stk[:,2],60, cmap='jet')
    cb=plt.colorbar(format='%.3f', orientation="vertical")
    p1,=plt.plot(bmod[0],bmod[1], 'k+', mew=5, ms=15,\
        label='Best Model %s km %s Vp/Vs'%(bmod[0], bmod[1]))
    plt.legend(bbox_to_anchor=(0., 1.02, 1., .102), loc=3,\
        ncol=2, mode="expand", borderaxespad=0, \
        handler_map={p1:HL(numpoints=1)}) 
    plt.ylabel('Vp/Vs')
    plt.xlabel('Depth km')
    plt.xlim(min(H),max(H))
    plt.ylim(min(K),max(K))
    # plt.savefig(savepath , format=format, dpi=250) 
    plt.show()
    plt.close("all")  
    Hbm = bmod[0]
    Kbm = bmod[1]
    Sbm = bmod[2]
    print("Best depth: ", Hbm, "Best Vp/Vs:", Kbm, "Max stack: ", Sbm)
    HKResult = {'RF': rfstream, 'RFStack': RFStacked, 'time':t, 'bestmodel':bmod, 'staname':staname, 'Vp':Vp,\
                'stackvalues':stk, 'layer':layer, 'stack':stack, 'comp':comp, 'AvgSlow':AvgSlow, 'H':H, 'K':K}
    return HKResult
#----------------------------------------------------------
def plothk(HKResult, savepath, g = [75.,10., 15., 2.5], rmneg = None, format = 'jpg'): 
    """
    Plot the H-K stacking results

    :param HKResult: Dictionary of H-K stacking results from hk function
    :type HKResult: dict
    :param savepath: path to save the figure
    :type savepath: str
    :param g: list of gain values for the plot
    :type g: list
    :param rmneg: remove the negative values from the stack (default True)
    :type rmneg: bool
    :param format: format of the figure
    :type format: str
    """
    if rmneg is None: rmneg = True
    if format is None: format = 'pdf'
    if g is None: g = [75.,10., 15., 2.5]
    staname=HKResult['staname']
    bmod = HKResult['bestmodel']
    Hbm = bmod[0]
    Kbm = bmod[1]
    Sbm = bmod[2]
    layer = HKResult['layer']
    comp=HKResult['comp']
    Vp=HKResult['Vp']
    if HKResult['stack'] == True:
        rfstream=HKResult['RFStack']
        rp = kilometer2degrees(1) * HKResult['AvgSlow']
        t=HKResult['time']
        term1= ((Kbm/Vp)**2 - rp**2)**0.5
        term2= ((1/Vp)**2 - rp**2)**0.5
        tps = Hbm * (term1 - term2)
        tppps = Hbm * (term1 + term2)
        tpsps = Hbm * 2 * (term1)
    else:
        rfstream=HKResult['RF']
        for tr in rfstream:
            rp = kilometer2degrees(1) * tr.stats['slowness']
            term1= ((Kbm/Vp)**2 - rp**2)**0.5
            term2= ((1/Vp)**2 - rp**2)**0.5
            tps = Hbm * (term1 - term2)
            tppps = Hbm * (term1 + term2)
            tpsps = Hbm * 2 * (term1)    
            tr.stats['tps'] = tps
            tr.stats['tppps'] = tppps
            tr.stats['tpsps'] = tpsps
    #----------------------------------------------------------
    # plot H-K results
    #----------------------------------------------------------
    if HKResult['layer'] == 'Moho':
        mintime=-5
        maxtime=30
        span=0.5
        timestep=5
    elif HKResult['layer'] == 'Sed':
        mintime=-1
        maxtime=10
        span=0.1
        timestep=1
    H=HKResult['H']
    K=HKResult['K']
    stk = HKResult['stackvalues']
    plt.figure(figsize=(18, 6)) 
    gs = gridspec.GridSpec(1, 3, width_ratios=[6,4,4])
    #-----Remove the stack values that is lower than zero------
    if rmneg == True:
        for i in range(len(stk)):
            if stk[i,2] <= 0:
                stk[i,2] = 0
    #----------------------------------------------------------
    ax1 = plt.subplot(gs[0, 0])
    plt.tricontourf(stk[:,0], stk[:,1], stk[:,2],50, cmap ="jet")
    #----------------------------------------------------------
    cb=plt.colorbar(format='%.3f', orientation="vertical")
    p1,=ax1.plot(bmod[0],bmod[1], 'k+', mew=2, ms=10, \
    label='%s Best Model %.2f km %.2f Vp/Vs'%(staname,bmod[0],bmod[1]+0.001))
    props = dict(boxstyle='round', facecolor='wheat', alpha=0.5)
    plt.legend(bbox_to_anchor=(0., 1.02, 1., .102), loc=3,ncol=2, mode="expand", borderaxespad=0, handler_map={p1:HL(numpoints=1)})
    ax1.set_ylabel('Vp/Vs')
    ax1.set_xlabel('Depth km')
    ax1.set_xlim(min(H), max(H))
    ax1.set_ylim(min(K), max(K))
    #----------------------------------------------------------
    # Plot the receiver function by Back Azimuth with the estimated times
    #----------------------------------------------------------
    ax2 = plt.subplot(gs[0, 1]) 
    if HKResult['stack'] == True:
        rfstream=HKResult['RFStack']
        baz=180
        rfdata =  ((rfstream) * g[0]) + baz
        major_ticks_x = np.arange(-10, 41, timestep)                                              
        minor_ticks_x = np.arange(-10, 41, 1) 
        major_ticks_y = np.arange(0, 361, 45)                                              
        minor_ticks_y = np.arange(-30, 390, 5)
        ax2.set_xticks(major_ticks_x)                                                       
        ax2.set_xticks(minor_ticks_x, minor=True)                                           
        ax2.set_yticks(major_ticks_y)                                                       
        ax2.set_yticks(minor_ticks_y, minor=True)                                          
        ax2.set_xlim(mintime, maxtime)
        ax2.set_ylim(-29, 390)
        ax2.plot(t, rfdata, "k-", lw=0.5)
        ax2.fill_between(t, baz, rfdata, where=rfdata > baz, facecolor='red', alpha = 0.25)
        ax2.fill_between(t, baz, rfdata, where=rfdata <= baz, facecolor='blue', alpha = 0.25)
        mtransforms.blended_transform_factory(ax2.transData, ax2.transAxes)
        #----------------------------------------------------------
        # Plot H-K sequential formula times
        #----------------------------------------------------------
        ampgain = g[1]
        ax2.plot([tps, tps], [baz+ampgain, baz-ampgain], 'g-', lw=1.5)
        ax2.plot([tppps, tppps], [baz+ampgain, baz-ampgain], 'g-', lw=1.5)
        ax2.plot([tpsps, tpsps], [baz+ampgain, baz-ampgain], 'g-', lw=1.5)
        ax2.axvspan(tps-span, tps+span, alpha=0.5, color='lightgreen')
        ax2.axvspan(tppps-span, tppps+span, alpha=0.5, color='lightgreen')
        ax2.axvspan(tpsps-span, tpsps+span, alpha=0.5, color='lightgreen')
        ax2.set_xlabel("Time (sec)")
        ax2.set_ylabel("Back-Azimuth (deg)") 
        filename = "%s/%s_%s_%s.%s"%(savepath, staname, comp, str(layer), format)
        plt.savefig(filename , format=format, transparent=False,\
        dpi=250, bbox_inches = 'tight', pad_inches=0.1)
        plt.show()
        plt.close('all')
    else:
        rfstream=HKResult['RF']
        gcarcarr = np.zeros(len(rfstream))
        for rf in rfstream:        
            delta = rf.stats.delta
            baz = rf.stats.back_azimuth
            b = rf.stats.starttime - rf.stats.onset
            tps = rf.stats.tps
            tppps = rf.stats.tppps
            tpsps = rf.stats.tpsps
            rfdata = ((rf.data) * g[0]) + baz
            t = (np.arange(0, len(rfdata), 1) * delta) + b
            major_ticks_x = np.arange(-10, 41, timestep)                                              
            minor_ticks_x = np.arange(-10, 41, 1) 
            major_ticks_y = np.arange(0, 361, 45)                                              
            minor_ticks_y = np.arange(-30, 390, 5)
            ax2.set_xticks(major_ticks_x)                                                       
            ax2.set_xticks(minor_ticks_x, minor=True)                                           
            ax2.set_yticks(major_ticks_y)                                                       
            ax2.set_yticks(minor_ticks_y, minor=True)                                          
            ax2.set_xlim(mintime, maxtime)
            ax2.set_ylim(-29, 390)
            ax2.plot(t, rfdata, "k-", lw=0.5)
            ax2.fill_between(t, baz, rfdata, where=rfdata > baz, facecolor='red', alpha = 0.25)
            ax2.fill_between(t, baz, rfdata, where=rfdata <= baz, facecolor='blue', alpha = 0.25)
            mtransforms.blended_transform_factory(ax2.transData, ax2.transAxes)
            #----------------------------------------------------------
            # Plot H-K sequential formula times
            #----------------------------------------------------------
            ampgain = g[1]
            ax2.plot([tps, tps], [baz+ampgain, baz-ampgain], 'g-', lw=1.5)
            ax2.plot([tppps, tppps], [baz+ampgain, baz-ampgain], 'g-', lw=1.5)
            ax2.plot([tpsps, tpsps], [baz+ampgain, baz-ampgain], 'g-', lw=1.5)
            #---------------------------------------------------------
            # Highlight specifiec time range on RF plot
            #----------------------------------------------------------
        ax2.axvspan(tps-span, tps+span, alpha=0.5, color='lightgreen')
        ax2.axvspan(tppps-span, tppps+span, alpha=0.5, color='lightgreen')
        ax2.axvspan(tpsps-span, tpsps+span, alpha=0.5, color='lightgreen')
        ax2.set_xlabel("Time (sec)")
        ax2.set_ylabel("Back-Azimuth (deg)")    
        #----------------------------------------------------------
        # Plot the receiver function by distance with the estimated times 
        #----------------------------------------------------------
        ax3 = plt.subplot(gs[0, 2])
        gcarcarr = np.zeros(len(rfstream))   
        for rf, j in zip(rfstream, np.arange(len(rfstream))):        
            delta = rf.stats.delta
            gcarc = rf.stats.distance
            gcarcarr[j] = gcarc
            b = rf.stats.starttime - rf.stats.onset
            tps = rf.stats.tps
            tppps = rf.stats.tppps
            tpsps = rf.stats.tpsps
            rfdata = ((rf.data) * g[2]) + gcarc
            t = (np.arange(0, len(rfdata), 1) * delta) + b    
            major_ticks_x = np.arange(-10, 41, timestep)                                              
            minor_ticks_x = np.arange(-10, 41, 1) 
            major_ticks_y = np.arange(30, 110, 10)                                              
            minor_ticks_y = np.arange(30, 111, 5)
            ax3.set_xticks(major_ticks_x)                                                       
            ax3.set_xticks(minor_ticks_x, minor=True)                                           
            ax3.set_yticks(major_ticks_y)                                                       
            ax3.set_yticks(minor_ticks_y, minor=True)                                          
            #----------------------------------------------------------
            # Plot grid on the figure
            #----------------------------------------------------------                                                              
            ax3.set_xlim(mintime, maxtime)
            ax3.set_ylim(25,110)
            ax3.plot(t, rfdata, "k-", lw=0.5)
            ax3.fill_between(t, gcarc, rfdata, where=rfdata > gcarc, facecolor='red', alpha = 0.25)
            ax3.fill_between(t, gcarc, rfdata, where=rfdata <= gcarc, facecolor='blue', alpha = 0.25)
            mtransforms.blended_transform_factory(ax3.transData, ax3.transAxes)
            #----------------------------------------------------------
            # Plot H-K sequential formula times
            #----------------------------------------------------------
            ampgain = g[3]
            ax3.plot([tps, tps], [gcarc+ampgain, gcarc-ampgain], 'g-', lw=1.5)
            ax3.plot([tppps, tppps], [gcarc+ampgain, gcarc-ampgain], 'g-', lw=1.5)
            ax3.plot([tpsps, tpsps], [gcarc+ampgain, gcarc-ampgain], 'g-', lw=1.5)
        #----------------------------------------------------------
        # Highlight specifiec time range on RF plot
        #----------------------------------------------------------
        ax3.axvspan(tps-span, tps+span, alpha=0.5, color='lightgreen')
        ax3.axvspan(tppps-span, tppps+span, alpha=0.5, color='lightgreen')
        ax3.axvspan(tpsps-span, tpsps+span, alpha=0.5, color='lightgreen')    
        ax3.set_xlabel("Time (sec)")
        ax3.set_ylabel("Distance (deg)")
        filename = "%s/%s_%s_%s.%s"%(savepath, staname, comp, str(layer), format)
        plt.savefig(filename , format=format, transparent=False,\
            dpi=250, bbox_inches = 'tight', pad_inches=0.1)
        plt.show()



"""
.. module:: SynthRF
        :synopsis: Create synthetic reciever function data and analyse the data for hk, sequential hk, 
        and resonance filtering
.. moduleauthor:: Stephen Akinremi <s.akinremi@utwente.nl> Islam Fadel <i.e.a.m.fadel@utwente.nl> (October 2023)
"""

import numpy as np
import matplotlib.pyplot as plt
from rfsed.SynRF.FwdRF import SynRF
import matplotlib.gridspec as gridspec
import matplotlib.transforms as mtransforms
from matplotlib.legend_handler import HandlerLine2D as HL
import seaborn
seaborn.set_style("darkgrid", {"axes.facecolor": ".85"})
from scipy import signal
from scipy.signal import argrelextrema, argrelmin, argrelmax
import math
import cmath
from scipy.fftpack import fft, ifft
#----------------------------------------------------------
def getamp(rfdata, tarray, t):
    """
    Get the amplitude of the receiver function at a specific time

    :param rfdata: receiver function data
    :type rfdata: numpy array
    :param tarray: time array
    :type tarray: numpy array
    :param t: time to get the amplitude
    :type t: float
    :return: amplitude of the receiver function at time t
    """
    amp = rfdata[(np.abs(tarray - t).argmin())]
    return amp
#----------------------------------------------------------
def hkSynth(Synthrf, time, rayp=0.04, H=np.linspace(20,60,201), K=np.linspace(1.65,1.95,121), 
            Vp=6.9, w1=0.6, w2=0.3, w3=0.1, layer = None):
    """
    Calculate the H-K stacking for synthetic receiver function after Zhu and Kanamori (2000)
    Zhu, L., & Kanamori, H. (2000). Moho depth variation in southern california from
    teleseismic receiver functions. Journal of Geophysical Research: Solid Earth, 105 , 
    2969-2980. doi: 10.1029/1999jb900322
    
    :param Synthrf: synthetic receiver function
    :type Synthrf: numpy array
    :param time: time array
    :type time: numpy array
    :param rayp: ray parameter
    :type rayp: float
    :param H: depth array
    :type H: numpy array
    :param K: Vp/Vs array
    :type K: numpy array
    :param Vp: Vp value
    :type Vp: float
    :param w1: weight for Ps
    :param w2: weight for PpPs
    :param w3: weight for PsPs+PpSs
    :type w1, w2, w3: float
    :param layer: layer name 'Moho' or 'Sed'
    :type layer: string
    :return: Dictionary of H-K stacking result for synthetic receiver function
    """
    if Vp is None: Vp = 6.5
    if w1 is None: w1 = 0.6
    if w2 is None: w2 = 0.3
    if w3 is None: w3 = 0.1   
    if layer is None: layer = 'Moho'
    #----------------------------------------------------------
    rp=rayp
    rfdata = Synthrf
    t = time
    stk = np.zeros((len(K)*len(H),3))
    z = 0 
    for i in range(len(K)):
        Ktemp = K[i]
        for j in range(len(H)):
            Htemp = H[j]
            s = 0.0
            trdata = rfdata
            #--------------------------------------
            term1= ((Ktemp/Vp)**2 - rp**2)**0.5
            term2= ((1/Vp)**2 - rp**2)**0.5
            tps = Htemp * (term1 - term2)
            tppps = Htemp * (term1 + term2)
            tpsps = Htemp * 2 * (term1) 
            #--------------------------------------
            ampps = getamp(trdata, t, tps)
            ampppps = getamp(trdata,t, tppps)
            amppsps = getamp(trdata, t, tpsps)
            #--------------------------------------
            stemp = (w1 * ampps) + (w2 * ampppps) - (w3 * amppsps)
            s = stemp + s
            stk[z,:] = [Htemp, Ktemp, s]
            z = z + 1
    bmod = stk[np.argmax(stk[:,2]),:]
    #Show the best model
    plt.tricontourf(stk[:,0], stk[:,1], stk[:,2],60, cmap='jet')
    cb=plt.colorbar(format='%.3f', orientation="vertical")
    p1,=plt.plot(bmod[0],bmod[1], 'k+', mew=5, ms=15,\
        label='Best Model %s km %s Vp/Vs'%(bmod[0], bmod[1]))
    plt.legend(bbox_to_anchor=(0., 1.02, 1., .102), loc=3,\
        ncol=2, mode="expand", borderaxespad=0, \
        handler_map={p1:HL(numpoints=1)}) 
    plt.ylabel('Vp/Vs')
    plt.xlabel('Depth km')
    plt.xlim(min(H),max(H))
    plt.ylim(min(K),max(K))
    # plt.savefig(savepath , format=format, dpi=250) 
    plt.show()
    plt.close("all")  
    Hbm = bmod[0]
    Kbm = bmod[1]
    Sbm = bmod[2]
    print("Best depth: ", Hbm, "Best Vp/Vs:", Kbm, "Max stack: ", Sbm)
    HKResultSynth = {'SynthRF': Synthrf, 'time':t, 'bestmodel':bmod, 'Vp':Vp, \
                'stackvalues':stk, 'layer':layer, 'rayp':rayp, 'H':H, 'K':K}
    return HKResultSynth
#----------------------------------------------------------
def hkSeqSynth(rfSynthSed, rfSynthMoho, time, rayp=0.04, HSed=np.linspace(0,10,201), KSed=np.linspace(1.65,2.25,201),
               VpSed=3.0, w1Sed=0.6, w2Sed=0.3, w3Sed=0.1, HMoho=np.linspace(20,60,201), KMoho=np.linspace(1.65,1.95,121),
               VpMoho=6.9, w1Moho=0.6, w2Moho=0.3, w3Moho=0.1, g = [75.,10., 15., 2.5], rmneg = None):
    """
    Calculate the Sequential H-K stacking for synthetic receiver function after Yeck et al., 2013
    Yeck, W. L., Sheehan, A. F., & Schulte-Pelkum, V. (2013, 6). Sequential h-k stacking to obtain accurate 
    crustal thicknesses beneath sedimentary basins. Bulletin of the Seismological Society of America, 103 , 2142-2150. doi: 10.1785/0120120290

    :param rfSynthSed: synthetic receiver function for sediment layer (high frequency)
    :param rfSynthMoho: synthetic receiver function for Moho layer (lower frequency)
    :type rfSynthSed, rfSynthMoho: numpy array
    :param time: time array
    :type time: numpy array
    :param rayp: ray parameter
    :type rayp: float
    :param HSed: depth array for sediment layer
    :type HSed: numpy array
    :param KSed: Vp/Vs array for sediment layer
    :type KSed: numpy array
    :param VpSed: Vp value for sediment layer
    :type VpSed: float
    :param w1Sed: weight for Ps
    :param w2Sed: weight for PpPs
    :param w3Sed: weight for PsPs+PpSs
    :type w1Sed, w2Sed, w3Sed: float
    :param HMoho: depth array for Moho layer
    :type HMoho: numpy array
    :param KMoho: Vp/Vs array for Moho layer
    :type KMoho: numpy array
    :param VpMoho: Vp value for Moho layer
    :type VpMoho: float
    :param w1Moho: weight for Ps
    :param w2Moho: weight for PpPs
    :param w3Moho: weight for PsPs+PpSs
    :type w1Moho, w2Moho, w3Moho: float
    :param g: gain for plotting
    :type g: list
    :param rmneg: remove negative values in the stack
    :type rmneg: boolean
    :return: Dictionary of Sequential H-K stacking result
    """
    if g is None: g = [75.,10., 15., 2.5]
    if VpSed is None: VpSed = 3.0
    if w1Sed is None: w1Sed = 0.6
    if w2Sed is None: w2Sed = 0.3
    if w3Sed is None: w3Sed = 0.1  
    if rmneg is None: rmneg = False
    rfdata = rfSynthMoho
    rp=rayp
    t = time
    #----------------------------------------------------------
    # Stack for Sediment Layer
    #----------------------------------------------------------
    stkSed = np.zeros((len(KSed)*len(HSed),3))
    z = 0 
    for i in range(len(KSed)):
        Ktemp = KSed[i]
        for j in range(len(HSed)):
            Htemp = HSed[j]
            s = 0.0
            trdata = rfSynthSed
            rp = rp
            #--------------------------------------
            term1= ((Ktemp/VpSed)**2 - rp**2)**0.5
            term2= ((1/VpSed)**2 - rp**2)**0.5
            tpsSed = Htemp * (term1 - term2)
            tpppsSed = Htemp * (term1 + term2)
            tpspsSed = Htemp * 2 * (term1) 
            #--------------------------------------
            ampps = getamp(trdata, t, tpsSed)
            ampppps = getamp(trdata,t, tpppsSed)
            amppsps = getamp(trdata, t, tpspsSed)
            #--------------------------------------
            stemp = (w1Sed * ampps) + (w2Sed * ampppps) - (w3Sed * amppsps)
            sSed = stemp + s
            stkSed[z,:] = [Htemp, Ktemp, sSed]
            z = z + 1
    bmodSed = stkSed[np.argmax(stkSed[:,2]),:]
    #----------------------------------------------------------
    # Plot Sediment Stacking results
    plt.tricontourf(stkSed[:,0], stkSed[:,1], stkSed[:,2],60, cmap='jet')
    cb=plt.colorbar(format='%.3f', orientation="vertical")
    p1,=plt.plot(bmodSed[0],bmodSed[1], 'k+', mew=5, ms=15,\
        label='Best Model %s km %s Vp/Vs'%(bmodSed[0], bmodSed[1]))
    plt.legend(bbox_to_anchor=(0., 1.02, 1., .102), loc=3,\
        ncol=2, mode="expand", borderaxespad=0, \
        handler_map={p1:HL(numpoints=1)}) 
    plt.ylabel('Vp/Vs')
    plt.xlabel('Depth km')
    plt.xlim(min(HSed),max(HSed))
    plt.ylim(min(KSed),max(KSed))
    # plt.savefig(savepath , format=format, dpi=250) 
    plt.show()
    plt.close("all")    
    #----------------------------------------------------------
    # Add Ps, PpPs, PsPs + PbSs timing to the data header
    #----------------------------------------------------------    
    H1 = bmodSed[0]
    K1 = bmodSed[1]
    SSed = bmodSed[2]
    print("Best Sediment depth: ", H1, "Best Sediment Vp/Vs:", K1, "Max stack: ", SSed)
    #----------------------------------------------------------
    # Stack for Moho Layer
    #----------------------------------------------------------
    stkMoho = np.zeros((len(KMoho)*len(HMoho),3))
    z = 0 
    for i in range(len(KMoho)):
        Ktemp = KMoho[i]
        for j in range(len(HMoho)):
            Htemp = HMoho[j]
            s = 0.0
            trdata = rfSynthMoho
            rp = rp
            #--------------------------------------
            term1 = ((K1/VpSed)**2-(rp)**2)**0.5
            term2 = ((1/VpSed)**2-(rp)**2)**0.5
            term3 = ((Ktemp/VpMoho)**2-(rp)**2)**0.5
            term4 = ((1/VpMoho)**2-(rp)**2)**0.5
            #--------------------------------------
            tpsMoho = (H1 * (term1 - term2)) + (Htemp * (term3 - term4))
            tpppsMoho = (H1 * (term1 + term2)) + (Htemp * (term3 + term4))
            tpspsMoho = (2 * H1 * term1) + (2 * Htemp * term3)
            #--------------------------------------
            ampps = getamp(trdata, t, tpsMoho) 
            ampppps = getamp(trdata, t, tpppsMoho)  
            amppsps = getamp(trdata, t, tpspsMoho)
            stemp = (w1Moho * ampps) + (w2Moho * ampppps) - (w3Moho * amppsps)
            sMoho = stemp + s
                #--------------------------------------
            stkMoho[z,:] = [Htemp, Ktemp, sMoho]
            z = z + 1
    bmodMoho = stkMoho[np.argmax(stkMoho[:,2]),:]
    #----------------------------------------------------------
    # plot Moho Stacking results
    for i in range(len(stkMoho)):
        if stkMoho[i,2] <= 0:
            stkMoho[i,2] = 0
    plt.tricontourf(stkMoho[:,0], stkMoho[:,1], stkMoho[:,2],60, cmap='jet')
    cb=plt.colorbar(format='%.3f', orientation="vertical")
    p1,=plt.plot(bmodMoho[0],bmodMoho[1], 'k+', mew=5, ms=15,\
        label='Best Model %s km %s Vp/Vs'%(bmodMoho[0], bmodMoho[1]))
    plt.legend(bbox_to_anchor=(0., 1.02, 1., .102), loc=3,\
        ncol=2, mode="expand", borderaxespad=0, \
        handler_map={p1:HL(numpoints=1)}) 
    plt.ylabel('Vp/Vs')
    plt.xlabel('Depth km')
    plt.xlim(min(HMoho),max(HMoho))
    plt.ylim(min(KMoho),max(KMoho))
    # plt.savefig(savepath , format=format, dpi=250) 
    plt.show()
    plt.close("all")    
    H2 = bmodMoho[0]
    K2 = bmodMoho[1] 
    print("Best Moho depth: ", H2, "Best Moho Vp/Vs:", K2, "Max stack: ", SSed)   
    SynthSeqHKResult = {'SynthRFSed': rfSynthSed, 'SynthRFMoho': rfSynthMoho, 'time':t, \
                          'bestmodelSed':bmodSed, 'stackvalueSed':stkSed, 'bestmodelMoho':bmodMoho, 'stackvalueMoho':stkMoho,
                            'RayP':rayp, 'VpSed':VpSed, 'VpMoho':VpMoho, 'HSed':HSed, 'KSed':KSed, 'HMoho':HMoho, 'KMoho':KMoho}
    return SynthSeqHKResult
#----------------------------------------------------------
def ResonanceFilt(Synthrf, time):
    """
    Resonance Filter for synthetic receiver function data (Removes sediment reverberation effect)

    :param Synthrf: synthetic receiver function data
    :type Synthrf: numpy array
    :param time: time array
    :type time: numpy array

    :return: A dictionary of input synthetic receiver function, Filtered receiver function, 
            autocorrelation of the data, resonance filter, time lag (2 way travel time of the sediment reverbration), 
            and the strength of the sediment reverbration (r)
    """
    t = time
    delta=t[1]-t[0]
    Dt=delta
    tzero=np.where(t==0)[0]
    tzero=int(tzero)  
    start=tzero
    rf=Synthrf[start::]
    t = t[start::]
    Dt=delta
    N=len(rf)
    fmax=1/(2*Dt)
    df=fmax/(N/2)
    #---------------------------------------------
    # Build the frequency vector
    N2=N/2
    #---------------------------------------------
    column1=np.arange(0,N2+1)    
    column2=np.arange(-N2+1,-1) 
    #---------------------------------------------
    # Join the two vectors
    full_column = np.concatenate((column1,column2))
    #---------------------------------------------
    f=df*full_column
    Nf = N/2+1
    dw=2*np.pi*df
    w=dw*full_column
    #----------------------------------------------------------------------------------------------------------------------
    # Do Auto-correlation
    #----------------------------------------------------------------------------------------------------------------------
    D=rf
    D=D-np.mean(D)
    D=signal.detrend(D, axis=-1, type='linear')
    #------------------------
    autoc=signal.correlate(D,D,mode='full')#,method='auto')
    autoc=autoc/max(autoc)
    autoc=autoc[N-1:2*N-1]
    #----------------------------------------------------------------------------------------------------------------------
    # Find time lag and amplitude of the autocorrelation
    #----------------------------------------------------------
    local_min=argrelmin(autoc)
    index=(local_min[0][0])
    r=(autoc[index])
    r=math.sqrt(r**2)
    tlag= t[index]
    #------------------------------------------------------------------------------
    # Build Filter
    #------------------------------------------------------------------------------
    resonanceflt = []
    for i in range(0, len(w)):
        flt= (1 + (r)*cmath.exp((-1j)*(w[i])*tlag))
        resonanceflt.append(flt)
    #------------------------------------------------------------------------------
    # Filter the Data with a Resonance Filter (Fourier Transform)
    F_trans=fft(rf)
    Filtered=F_trans*resonanceflt
    F_inv=ifft(Filtered)
    #Use Real Part of the Inverse Fourier Transform
    filtered_rf=np.real(F_inv)  #real part of the inverse fourier transform
    #------------------------------------------------------------------------------
    FltResults={'rf':rf, 'filteredrf': filtered_rf, 'resonancefilter':resonanceflt, 'time':t, 'autoc':autoc, 'r':r, 'tlag':tlag, 'delta':delta}
    return FltResults

def plothkSynth(HKResultSynth, savepath, g = [75.,10., 15., 2.5], rmneg = None, format = 'jpg'): 
    """
    Plot the H-K stacking result for synthetic receiver function

    :param HKResultSynth: Dictionary containing H-K stacking result from function hkSynth
    :type HKResultSynth: dictionary
    :param g: gain for plotting
    :type g: list
    :param rmneg: remove negative values in the stack
    :type rmneg: boolean
    :param format: format for saving the figure
    :type format: string
    :param savepath: path to save the figure
    :type savepath: string
    """
    if rmneg is None: rmneg = False
    if format is None: format = 'pdf'
    if g is None: g = [75.,10., 15., 2.5]
    staname = 'SynthRF'
    bmod = HKResultSynth['bestmodel']
    Hbm = bmod[0]
    Kbm = bmod[1]
    Sbm = bmod[2]
    layer = HKResultSynth['layer']
    rfstream=HKResultSynth['SynthRF']
    rp = HKResultSynth['rayp']
    t=HKResultSynth['time']
    Vp=HKResultSynth['Vp']
    term1= ((Kbm/Vp)**2 - rp**2)**0.5
    term2= ((1/Vp)**2 - rp**2)**0.5
    tps = Hbm * (term1 - term2)
    tppps = Hbm * (term1 + term2)
    tpsps = Hbm * 2 * (term1)
    #----------------------------------------------------------
    # plot H-K results
    #----------------------------------------------------------
    if HKResultSynth['layer'] == 'Moho':
        mintime=-5
        maxtime=30
        span=0.5
        timestep=5
    elif HKResultSynth['layer'] == 'Sed':
        mintime=-1
        maxtime=10
        span=0.1
        timestep=1
    stk = HKResultSynth['stackvalues']
    H = HKResultSynth['H']
    K = HKResultSynth['K']
    plt.figure(figsize=(18, 6)) 
    gs = gridspec.GridSpec(1, 3, width_ratios=[6,4,4])
    #-----Remove the stack values that is lower than zero------
    if rmneg == True:
        for i in range(len(stk)):
            if stk[i,2] <= 0:
                stk[i,2] = 0
    #----------------------------------------------------------
    ax1 = plt.subplot(gs[0, 0])
    plt.tricontourf(stk[:,0], stk[:,1], stk[:,2],50, cmap ="jet")
    #----------------------------------------------------------
    cb=plt.colorbar(format='%.3f', orientation="vertical")
    p1,=ax1.plot(bmod[0],bmod[1], 'k+', mew=2, ms=10, \
    label='%s Best Model %.2f km %.2f Vp/Vs'%(staname,bmod[0],bmod[1]+0.001))
    props = dict(boxstyle='round', facecolor='wheat', alpha=0.5)
    plt.legend(bbox_to_anchor=(0., 1.02, 1., .102), loc=3,ncol=2, mode="expand", borderaxespad=0, handler_map={p1:HL(numpoints=1)})
    ax1.set_ylabel('Vp/Vs')
    ax1.set_xlabel('Depth km')
    ax1.set_xlim(min(H), max(H))
    ax1.set_ylim(min(K), max(K))
    #----------------------------------------------------------
    # Plot the receiver function by Back Azimuth with the estimated times
    #----------------------------------------------------------
    ax2 = plt.subplot(gs[0, 1]) 
    gcarcarr = np.zeros(len(rfstream))
    baz = 180
    rfdata = ((rfstream) * g[0]) + baz
    t = t
    major_ticks_x = np.arange(-10, 41, timestep)                                              
    minor_ticks_x = np.arange(-10, 41, 1) 
    major_ticks_y = np.arange(0, 361, 45)                                              
    minor_ticks_y = np.arange(-30, 390, 5)
    ax2.set_xticks(major_ticks_x)                                                       
    ax2.set_xticks(minor_ticks_x, minor=True)                                           
    ax2.set_yticks(major_ticks_y)                                                       
    ax2.set_yticks(minor_ticks_y, minor=True)                                          
    ax2.set_xlim(mintime, maxtime)
    ax2.set_ylim(-29, 390)
    ax2.plot(t, rfdata, "k-", lw=0.5)
    ax2.fill_between(t, baz, rfdata, where=rfdata > baz, facecolor='red', alpha = 0.25)
    ax2.fill_between(t, baz, rfdata, where=rfdata <= baz, facecolor='blue', alpha = 0.25)
    mtransforms.blended_transform_factory(ax2.transData, ax2.transAxes)
    #----------------------------------------------------------
    # Plot H-K sequential formula times
    #----------------------------------------------------------
    ampgain = g[1]
    ax2.plot([tps, tps], [baz+ampgain, baz-ampgain], 'g-', lw=1.5)
    ax2.plot([tppps, tppps], [baz+ampgain, baz-ampgain], 'g-', lw=1.5)
    ax2.plot([tpsps, tpsps], [baz+ampgain, baz-ampgain], 'g-', lw=1.5)
    #---------------------------------------------------------
    # Highlight specifiec time range on RF plot
    #----------------------------------------------------------
    ax2.axvspan(tps-span, tps+span, alpha=0.5, color='lightgreen')
    ax2.axvspan(tppps-span, tppps+span, alpha=0.5, color='lightgreen')
    ax2.axvspan(tpsps-span, tpsps+span, alpha=0.5, color='lightgreen')
    ax2.set_xlabel("Time (sec)")
    ax2.set_ylabel("Back-Azimuth (deg)")    
    #----------------------------------------------------------
    filename = "%s/%s_%s.%s"%(savepath, staname, str(layer), format)
    plt.savefig(filename , format=format, transparent=False,\
        dpi=250, bbox_inches = 'tight', pad_inches=0.1)
    plt.show()
    plt.close('all')
#----------------------------------------------------------
def plothkSeqSynth(SynthSeqHKResult, savepath, g = [75.,10., 15., 2.5], rmneg = None, format = 'jpg'): 
    """
    Plot the Sequential H-K stacking result for synthetic receiver function

    :param SynthSeqHKResult: Dictionary containing Sequential H-K stacking result from function hkSeqSynth
    :type SynthSeqHKResult: dictionary
    :param g: gain for plotting
    :type g: list
    :param rmneg: remove negative values in the stack
    :type rmneg: boolean
    :param format: format for saving the figure
    :type format: string
    :param savepath: path to save the figure
    :type savepath: string
    """
    if rmneg is None: rmneg = False
    if format is None: format = 'pdf'
    if g is None: g = [75.,10., 15., 2.5]
    staname='SynthRF_SeqHK'
    VpSed=SynthSeqHKResult['VpSed']
    HSed=SynthSeqHKResult['HSed']
    KSed=SynthSeqHKResult['KSed']
    bmodSed = SynthSeqHKResult['bestmodelSed']
    H1 = bmodSed[0]
    K1 = bmodSed[1]
    stkSed=SynthSeqHKResult['stackvalueSed']
    
    rfstream=SynthSeqHKResult['SynthRFSed']
    rp = SynthSeqHKResult['RayP']
    t=SynthSeqHKResult['time']
    term1= ((K1/VpSed)**2 - rp**2)**0.5
    term2= ((1/VpSed)**2 - rp**2)**0.5
    tpsSed = H1 * (term1 - term2)
    tpppsSed = H1 * (term1 + term2)
    tpspsSed = H1 * 2 * (term1)
    #----------------------------------------------------------
    # plot H-K results
    #----------------------------------------------------------
    plt.figure(figsize=(18, 6)) 
    gs = gridspec.GridSpec(1, 3, width_ratios=[6,4,4])
    #----------------------------------------------------------
    # plot misfit
    #----------------------------------------------------------
    #-----Remove the stack values that is lower than zero------
    if rmneg == True:
        for i in range(len(stkSed)):
            if stkSed[i,2] <= 0:
                stkSed[i,2] = 0
    #----------------------------------------------------------
    ax1 = plt.subplot(gs[0, 0])
    plt.tricontourf(stkSed[:,0], stkSed[:,1], stkSed[:,2],50, cmap ="jet")
    #----------------------------------------------------------
    cb=plt.colorbar(format='%.3f', orientation="vertical")
    p1,=ax1.plot(bmodSed[0],bmodSed[1], 'k+', mew=2, ms=10, \
    label='%s Best Model %.2f km %.2f Vp/Vs'%(staname,bmodSed[0],bmodSed[1]+0.001))
    props = dict(boxstyle='round', facecolor='wheat', alpha=0.5)

    plt.legend(bbox_to_anchor=(0., 1.02, 1., .102), loc=3,ncol=2, mode="expand", borderaxespad=0, handler_map={p1:HL(numpoints=1)})
    ax1.set_ylabel('Vp/Vs')
    ax1.set_xlabel('Depth km')
    ax1.set_xlim(min(HSed), max(HSed))
    ax1.set_ylim(min(KSed), max(KSed))
    #----------------------------------------------------------
    # Plot the receiver function witht the estimated times
    #----------------------------------------------------------
    ax2 = plt.subplot(gs[0, 1]) 
    baz=180
    rfdata = ((SynthSeqHKResult['SynthRFSed']) * (g[0]/2)) + baz
    major_ticks_x = np.arange(-10, 41, 1)                                              
    minor_ticks_x = np.arange(-10, 41, 1) 
    major_ticks_y = np.arange(0, 361, 45)                                              
    minor_ticks_y = np.arange(-30, 390, 5)
    ax2.set_xticks(major_ticks_x)                                                       
    ax2.set_xticks(minor_ticks_x, minor=True)                                           
    ax2.set_yticks(major_ticks_y)                                                       
    ax2.set_yticks(minor_ticks_y, minor=True)                                          
    ax2.set_xlim(-1, 10)
    ax2.set_ylim(-29, 390)
    ax2.set_xlabel("Time (sec)")
    ax2.set_ylabel("Back-Azimuth (deg)")  
    ax2.plot(t, rfdata, "k-", lw=0.5)
    ax2.fill_between(t, baz, rfdata, where=rfdata > baz, facecolor='red', alpha = 0.25)
    ax2.fill_between(t, baz, rfdata, where=rfdata <= baz, facecolor='blue', alpha = 0.25)
    mtransforms.blended_transform_factory(ax2.transData, ax2.transAxes)
    ampgain = g[1]
    ax2.plot([tpsSed, tpsSed], [baz+ampgain, baz-ampgain], 'g-', lw=1.5, label="Moho")
    ax2.plot([tpppsSed, tpppsSed], [baz+ampgain, baz-ampgain], 'g-', lw=1.5)
    ax2.plot([tpspsSed, tpspsSed], [baz+ampgain, baz-ampgain], 'g-', lw=1.5)
    filename = "%s/%s_%s.%s"%(savepath, staname, "Sediment", format)
    plt.savefig(filename , format=format, transparent=False,\
        dpi=250, bbox_inches = 'tight', pad_inches=0.1)
    plt.show()
    plt.close('all')
    #----------------------------------------------------------
    # Plot Moho Stack
    #----------------------------------------------------------
    VpMoho=SynthSeqHKResult['VpMoho']
    HMoho=SynthSeqHKResult['HMoho']
    KMoho=SynthSeqHKResult['KMoho']
    bmodMoho = SynthSeqHKResult['bestmodelMoho']  
    H2 = bmodMoho[0]
    K2 = bmodMoho[1]
    stkMoho=SynthSeqHKResult['stackvalueMoho']
    rfstream=SynthSeqHKResult['SynthRFMoho']
    rp = rp
    t=SynthSeqHKResult['time']
    term1 = ((K1/VpSed)**2-(rp)**2)**0.5    
    term2 = ((1/VpSed)**2-(rp)**2)**0.5
    term3 = ((K2/VpMoho)**2-(rp)**2)**0.5    
    term4 = ((1/VpMoho)**2-(rp)**2)**0.5
    #--------------------------------------
    tpsMoho = (H1 * (term1 - term2)) + (H2 * (term3 - term4))
    tpppsMoho = (H1 * (term1 + term2)) + (H2 * (term3 + term4))
    tpspsMoho = (2 * H1 * term1) + (2 * H2 * term3)
    #----------------------------------------------------------
    plt.figure(figsize=(18, 6)) 
    gs = gridspec.GridSpec(1, 3, width_ratios=[6,4,4])
    #-----Remove the stack values that is lower than zero------
    if rmneg == True:
        for i in range(len(stkMoho)):
            if stkMoho[i,2] <= 0:
                stkMoho[i,2] = 0
    #----------------------------------------------------------
    ax1 = plt.subplot(gs[0, 0])
    plt.tricontourf(stkMoho[:,0], stkMoho[:,1], stkMoho[:,2],50, cmap ="jet")
    cb=plt.colorbar(format='%.3f', orientation="vertical")
    cb.ax.set_xlabel('S', rotation=0)
    p1,=ax1.plot(bmodMoho[0],bmodMoho[1], 'k+', mew=2, ms=10, \
    label='%s Best Model %.2f km %.2f Vp/Vs'%(staname,bmodMoho[0],bmodMoho[1]+0.001))
    props = dict(boxstyle='round', facecolor='wheat', alpha=0.5)
    plt.legend(bbox_to_anchor=(0., 1.02, 1., .102), loc=3,\
        ncol=2, mode="expand", borderaxespad=0, \
        handler_map={p1:HL(numpoints=1)})
    ax1.set_ylabel('Vp/Vs')
    ax1.set_xlabel('Depth km')
    ax1.set_xlim(min(HMoho), max(HMoho))
    ax1.set_ylim(min(KMoho), max(KMoho))
    #----------------------------------------------------------
    # Plot the receiver function witht the estimated times
    #----------------------------------------------------------
    ax2 = plt.subplot(gs[0, 1]) 
    baz=180
    rfdata = ((SynthSeqHKResult['SynthRFMoho']) * g[0]) + baz
    major_ticks_x = np.arange(-10, 41, 5)                                           
    minor_ticks_x = np.arange(-10, 41, 1) 
    major_ticks_y = np.arange(0, 361, 45)                                              
    minor_ticks_y = np.arange(-30, 390, 5)
    ax2.set_xticks(major_ticks_x)                                                       
    ax2.set_xticks(minor_ticks_x, minor=True)                                           
    ax2.set_yticks(major_ticks_y)                                                       
    ax2.set_yticks(minor_ticks_y, minor=True)                                  
    ax2.set_xlim(-5, 30)
    ax2.set_ylim(-29, 390)
    ax2.plot(t, rfdata, "k-", lw=0.5)
    ax2.fill_between(t, baz, rfdata, where=rfdata > baz, 
                        facecolor='red', alpha = 0.25)
    ax2.fill_between(t, baz, rfdata, where=rfdata <= baz, 
                        facecolor='blue', alpha = 0.25)
    mtransforms.blended_transform_factory(ax2.transData, ax2.transAxes)
    #----------------------------------------------------------
    # Plot H-K sequential formula times
    #----------------------------------------------------------
    ampgain = g[1]
    ax2.plot([tpsMoho, tpsMoho], [baz+ampgain, baz-ampgain], 'g-', lw=1.5, label="Moho")
    ax2.plot([tpppsMoho, tpppsMoho], [baz+ampgain, baz-ampgain], 'g-', lw=1.5)
    ax2.plot([tpspsMoho, tpspsMoho], [baz+ampgain, baz-ampgain], 'g-', lw=1.5)
    filename = "%s/%s_%s.%s"%(savepath, staname, 'Moho', format)
    plt.savefig(filename , format=format, transparent=False,\
    dpi=250, bbox_inches = 'tight', pad_inches=0.1)
    plt.show()
    plt.close('all')
#----------------------------------------------------------
def plotfiltSynthrf(FilteredRF, savepath, format = 'jpg'):
    """
    Plot the filtered receiver function, autocorrelation, and the resonance filter

    :param FilteredRF: Dictionary of results from the resonance filtering method (function: Resonance_Filt)
    :type FilteredRF: dictionary
    :param savepath: path to save the plots
    :type savepath: str
    :param format: format of the plot (default: jpg)
    :type format: str
    """
    rf=FilteredRF['rf']
    filtered_rf=FilteredRF['filteredrf']
    resonanceflt=FilteredRF['resonancefilter']
    time=FilteredRF['time']
    autoc=FilteredRF['autoc']
    suff='SyntheticRF'
    # Plot Autocorrelation
    plt.plot(time,autoc)
    plt.xlabel('Time (s)')
    plt.ylabel('Amplitude')
    plt.title('Autocorrelation of %s'%suff)
    filename = "%s/%s.%s"%(savepath, 'Autocorrelation', format)
    plt.savefig(filename , format=format, transparent=False,\
            dpi=250, bbox_inches = 'tight', pad_inches=0.1)
    plt.show()
    plt.close("all")
    # Plot Resonance Filter
    plt.plot(time, resonanceflt)
    plt.xlabel('Time (s)')
    # plt.ylabel('Amplitude')
    plt.title('Resonance Filter of %s'%suff)
    filename = "%s/%s.%s"%(savepath, "Resonance_Filter", format)
    plt.savefig(filename , format=format, transparent=False,\
            dpi=250, bbox_inches = 'tight', pad_inches=0.1)
    plt.show()
    plt.close("all")
    # Plot RF and Filtered RF
    plt.plot(time, rf, label = 'RF', color = 'black')
    plt.plot(time, filtered_rf, label = 'Filtered RF', color = 'red')
    plt.xlabel('Time (s)')
    plt.ylabel('Amplitude')
    plt.title('RF and Filtered RF of %s'%suff)
    plt.legend()
    filename = "%s/%s.%s"%(savepath, "RF_FilteredRF", format)
    plt.savefig(filename , format=format, transparent=False,\
            dpi=250, bbox_inches = 'tight', pad_inches=0.1)
    plt.show()
    plt.close("all")
#----------------------------------------------------------


from rfsed.rf.rfstream import read_rf, rfstats
from pkg_resources import resource_filename
import numpy as np
import os


def rfMoho_example():
    """
    Return low-frequency receiver functions in a stream by read_rf().
    """
    fname = resource_filename('rfsed', 'data/rfstreams_Moho/rfstreams.h5')
    stream = read_rf(fname)  
    return stream

def rfSed_example():
    """
    Return high-frequency receiver functions in a stream read_rf().
    """
    fname = resource_filename('rfsed', 'data/rfstreams_Sed/rfstreams.h5')
    stream = read_rf(fname)  
    return stream

def raw_data_example():
    """
    Return directory to raw seismic record data.
    """
    data_dir = resource_filename('rfsed', 'data/Raw_Data_DeepNL_Groningen/')

    return data_dir

def catalog_example():
    """
    Return directory to earthquake catalog.
    """
    catalog_dir = resource_filename('rfsed', 'data/Earthquake_Catalog/eq_events.quakeml')
    return catalog_dir

def save_Eq_data():
    """
    Return directory to save extracted earthquake data.
    """
    savepath = resource_filename('rfsed', 'examples/earthquake_data/')
    if not os.path.exists(savepath):
        os.mkdir(savepath)
    return savepath

def save_catalog():
    """
    Return directory to save extracted earthquake data.
    """
    savepath = resource_filename('rfsed', 'examples/catalog/')
    if not os.path.exists(savepath):
        os.mkdir(savepath)
    return savepath

def save_plot():
    """
    Return directory to save extracted earthquake data.
    """
    savepath = resource_filename('rfsed', 'examples/plots/')
    if not os.path.exists(savepath):
        os.mkdir(savepath)
    return savepath

def save_tests():
    """
    Return directory to save test plots.
    """
    savepath = resource_filename('rfsed', 'tests/plot/')
    if not os.path.exists(savepath):
        os.mkdir(savepath)
    return savepath


#%% Test the Waveform Fitting Function
import os 
from os.path import exists
from os import mkdir
import numpy as np
from glob import glob
from rfsed.rf import read_rf, RFStream
from rfsed.WaveformFitting import WaveformFitting, PlotWaveformFitting    
from rfsed.util import rfMoho_example, rfSed_example, save_plot

#----------------------------------------------------------
savedir=save_plot()
savepath = savedir + '/WaveformFitting/'
if not os.path.exists(savepath):  # create data folder if necessary
    os.mkdir(savepath)

rfstream= rfMoho_example()
VpSed=2.2
HSed=0.1
VpCrust = 6.9
gaussian=1.25
rayp=0.04
KMoho= np.linspace(1.65,1.95,121)
HMoho=np.linspace(20,60,201)
# KMoho= np.linspace(1.65,1.95,5)
# HMoho=np.linspace(20,60,5)
wtCorr, wtRMSE, wtPG = [0.5, 0.3, 0.2] #wtCorr+wtRMSE+wtPG=1.0
format='pdf'
FittingResult = WaveformFitting(rfstream, HSed, VpSed, VpCrust, rayp, KMoho, HMoho, 
                     gaussian, wtCorr, wtRMSE, wtPG, savepath, format)
#%%
PlotWaveformFitting(FittingResult, wtCorr, wtRMSE, wtPG, savepath, format)
#%%


# Copyright 2013-2016 Tom Eulenfeld, MIT license
"""
Tests for imaging module.
"""

import unittest
import warnings

from rfsed.rf.util import minimal_example_rf

try:
    import cartopy
except ImportError:
    cartopy = None


class ImagingTestCase(unittest.TestCase):

    def test_plot_rf(self):
        stream = minimal_example_rf()
        streamQ = stream.select(component='Q')
        streamQ.plot_rf()
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            stream.plot_rf()
        # test plotting with different number of samples
        streamQ[0].data = streamQ[0].data[:-10]
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            streamQ.plot_rf()

    @unittest.skipIf(cartopy is None, 'cartopy not installed')
    def test_plot_ppoints(self):
        from rfsed.rf.imaging import plot_ppoints
        stream = minimal_example_rf()
        plot_ppoints(stream.ppoints(50), inventory=stream)


def suite():
    return unittest.makeSuite(ImagingTestCase, 'test')


if __name__ == '__main__':
    unittest.main(defaultTest='suite')



import setuptools
from setuptools import setup
from setuptools.command.test import test as testcommand

with open("./README.md", "r") as fh:
    long_description = fh.read()
REQUIRES = ['decorator', 'matplotlib', 'numpy', 'scipy',
             'obspy>=1.0.3', 'tqdm', 'OWSLib', 'pytest',
            'cartopy', 'geographiclib', 'shapely', 'seaborn',
            'scipy', 'scikit-learn', 'h5','h5py', 'obspyh5', 
            'colorama==0.1.6', 'plotly==5.17.0', 'psutil==5.9.6']
CLASSIFIERS = [
    'Environment :: Console',
    'Intended Audience :: Science/Research',
    'License :: OSI Approved :: MIT License',
    'Operating System :: OS Independent',
    'Programming Language :: Python :: 3',
    'Topic :: Scientific/Engineering :: Physics'
    ]
EXTRAS_REQUIRE = {
    'doc': ['sphinx', 'alabaster'], 
    'deconv_multitaper': ['mtspec'],
    'h5': ['obspyh5>=0.3'],
    'toeplitz': ['toeplitz'],
    'batch': ['tqdm']}
setuptools.setup(
    name="rfsed",
    version="0.0.2",
    author='Stephen Akinremi',
    author_email='s.akinremi@utwente.nl',
    description="Package for receiver functions analysis and dealing with sediment effects in receiver functions",
    long_description=long_description,
    url="https://github.com/akinremisa/rfsed",
    packages=setuptools.find_packages(),
    setup_requires=REQUIRES,
    install_requires=REQUIRES,
    extras_require=EXTRAS_REQUIRE,
    classifiers=CLASSIFIERS,
    python_requires='>=3.6'
)

class PyTest(testcommand):
    user_options = [('pytest-args=', 'a', "Arguments to pass to py.tests")]

    def initialize_options(self):
        testcommand.initialize_options(self)
        self.pytest_args = []

    def run_tests(self):
        import pytest
        import sys
        errno = pytest.main(self.pytest_args)
        sys.exit(errno)


setup(cmdclass={'tests': PyTest})


# Copyright 2013-2016 Tom Eulenfeld, MIT license
from obspy import read_events, read_inventory, Stream, UTCDateTime as UTC
from obspy.clients.fdsn import Client

from rfsed.rf.rfstream import obj2stats, rfstats
from rfsed.rf import RFStream


evname = './example_events.xml'
invname = './example_inventory.xml'
wavname = './example_data.mseed'
wavname2 = './minimal_example.sac'

lon, lat = -70, -21
t1, t2 = UTC('2011-02-01'), UTC('2011-06-01')
seedid = 'CX.PB01..BH?'


def get_events():
    try:
        return read_events(evname)
    except Exception:
        pass
    client = Client()
    events = client.get_events(starttime=t1, endtime=t2, latitude=lat,
                               longitude=lon, minradius=30, maxradius=90,
                               minmagnitude=6., maxmagnitude=6.5)
    events.write(evname, 'QUAKEML')
    return events


def get_inventory():
    try:
        return read_inventory(invname)
    except Exception:
        pass
    client = Client('GFZ')
    net, sta, loc, cha = seedid.split('.')
    inv = client.get_stations(starttime=t1, endtime=t2, network=net,
                              station=sta, location=loc, channel=cha,
                              level='channel')
#                               latitude=lat, longitude=lon, maxradius=10)
    inv.write(invname, 'STATIONXML')
    return inv


def get_waveforms():
    events = get_events()[::-1]
    client = Client('GFZ')
    stream_raw = Stream()
    stream = RFStream()
    coords = inventory.get_coordinates(seedid[:-1] + 'Z')
    for i, event in enumerate(events):
        t = event.preferred_origin().time
        args = seedid.split('.') + [t + 4.9 * 60, t + 14.1 * 60]
        s = client.get_waveforms(*args)
        s.trim(t+5*60, t+14*60)
        s.decimate(int(round(s[0].stats.sampling_rate)) // 5, no_filter=True)
        stream_raw.extend(s)
        if i in (0, 2, 4):
            s = s.copy()
            stats = rfstats(station=coords, event=event, dist_range=(20, 95))
            if stats is None:
                continue
            s.trim(stats.onset - 25, stats.onset + 75)
            stats = obj2stats(station=coords, event=event)
            s = RFStream(s)
            for tr in s:
                tr.stats.update(stats)
            stream.extend(s)
    stream_raw.write(wavname, 'MSEED')
    stream.write(wavname2, 'SAC')

inventory = get_inventory()
events = get_events()
get_waveforms()
